<?php

namespace Alive\Dm;

class Dm_Utils {

    /**
     * Check whether the given discount is active.
     * * Status is active.
     * * Valid since is in past.
     * * Valid until is in future or is not set.
     *
     * @param StdObj $discount Discount to be checked.
     * @return boolean True if given discount is active, false otherwise.
     */
    public static function is_discount_active($discount) {
        if ($discount->active != true) {
            return false;
        }
        $validSince = new \DateTime($discount->validSince);
        $validUntil = !empty($discount->validUntil) ? new \DateTime($discount->validUntil) : null;
        if ($validSince > new \DateTime()) {
            return false;
        }
        if ($validUntil !== null && $validUntil < new \DateTime()) {
            return false;
        }
        return true;
    }

    /**
     * Check whenther the branch is active.
     * * Status is active.
     * * Has at least one active discount.
     *
     * @param StdObj $branch Branch to be checked. Never null.
     * @param StdObh $provider Provider of the branch.
     * @return booleanTrue if given branch is active, false otherwise.
     */
    public static function is_branch_active($branch, $provider) {
        if ($branch->active != true) {
            return false;
        }
        foreach ($provider->discounts as $discount) {
            foreach ($discount->branches as $discountBranch) {
                if ($branch->id == $discountBranch->id && self::is_discount_active($discount)) {
                    return true;
                }
            }
        }
        return false;
    }

}
