<?php

namespace Alive\Dm;

class Localizer {

    /**
     * Get provider localized value.
     * @param object $provider Provider object. Never null.
     * @param string$fieldName Name of the field to be returned.
     */
    public static function get_provider_value($provider, $fieldName) {
        $l10n = self::getL10n($provider->providerL10ns);
        if ($l10n != null && isset($l10n->{$fieldName}) && $l10n->{$fieldName} != null) {
            return $l10n->{$fieldName};
        }
        return $provider->{$fieldName};
    }

    /**
     * Get discount localized value.
     * @param object $discount DIscount object. Never null.
     * @param string$fieldName Name of the field to be returned.
     */
    public static function get_discount_value($discount, $fieldName) {
        $l10n = self::getL10n($discount->discountL10ns);
        if ($l10n != null && isset($l10n->{$fieldName}) && $l10n->{$fieldName} != null) {
            return $l10n->{$fieldName};
        }
        return $discount->{$fieldName};
    }

    /**
     * Get branch localized value.
     * @param object $branch Branch object. Never null.
     * @param string$fieldName Name of the field to be returned.
     */
    public static function get_branch_value($branch, $fieldName) {
        $l10n = self::getL10n($branch->branchL10ns);
        if ($l10n != null && isset($l10n->{$fieldName}) && $l10n->{$fieldName} != null) {
            return $l10n->{$fieldName};
        }
        return $branch->{$fieldName};
    }

    /**
     * Get localization object for actual locale. Assumes that l10n object has field 'languageCode'.
     * @param array $l10ns List of localization objects to be searched.
     */
    private static function getL10n($l10ns) {
        $locale = Config::locale();
        foreach ((array) $l10ns as $l10n) {
            // Check whether the translation in provided language exists.
            if (strtolower($l10n->languageCode) == strtolower(substr($locale, 0, 2))) {
                return $l10n;
            }
        }
        return null;
    }

    /**
     * Get localized name of the entity.
     * @param category $category Category enetity including l10ns.
     */
    public static function get_category_name($category) {
        $l10n = self::getL10n($category->categoryL10ns);
        if (!empty($l10n->name)) {
            return $l10n->name;
        }
        // Fallback to untranslated data.
        return $category->name;
    }

    public static function get_discount_type($discount) {
        switch ($discount->type) {
            case 'ABSOLUTE':
                return self::returnAsMoney($discount->value) . ' ' . __dm('discount');
            case 'PERCENTAGE':
                return $discount->value . '% ' . __dm('discount');
            case 'FREE_1PLUS1':
                return __dm('1 + 1 free');
            case 'FREE_2PLUS1':
                return __dm('2 + 1 free');
            case 'FREE_3PLUS1':
                return __dm('3 + 1 free');
            case 'FREE_4PLUS1':
                return __dm('4 + 1 free');
            case 'FREE_SHIPPING':
                return __dm('Free shipping');
            case 'FREE_GIFT':
                return __dm('Free gifr');
            case 'SPECIAL':
                return __dm('Special offer');
            default:
                return $discount->type;
        }
    }

    private static function returnAsMoney($amount) {
        if (Config::currency_pos() == 1) {
            return Config::currency() . ' ' . number_format_i18n($amount);
        } else if (Config::currency_pos() == 2) {
            return number_format_i18n($amount) . ' ' . Config::currency();
        } else {
            throw new Exception("Should not get here.");
        }
    }

}
