(function( $ ) {
    "use strict";

    /**
     * Class responsible for handling of the discount finder.
     */
    window.DmDiscountFinder = function(wrapper, config) {
        this.constructor.call(this, wrapper, config);
    }
    $.extend(DmDiscountFinder.prototype, {

        /**
         * Wrapper element of the discount finder DOM.
         */
        queryParamsPrefix: 'dm',

        /**
         * Wrapper element of the discount finder DOM.
         */
        wrapper: null,

        /**
         * Base URL. Should be replaced by client.
         */
        ajaxUrl: null,

        /**
         * Wrapper of the element containig search results.
         */
        resultsWrapper: null,

        /**
         * Wrapper of th results shown on map.
         */
        mapResultsWrapper: null,

        /**
         * Wraper of elements that should be replaced by the content.
         */
        contentWrapper: null,

        /**
         * Default page sizes for the lists.
         */
        pageSize: {
            list: 20,
            images: 10
        },

        /**
         * Curent display of the results.
         */
        display: null,

        /**
         * Current providerId.
         */
        providerId: null,

        /**
         * Current provider.
         */
        provider: null,

        /**
         * Class responsible for rendering list.
         */
        listRenderer: null,

        /**
         * Class responsible for renderign images view.
         */
        imagesRenderer: null,

        /**
         * Class responsible for rendering provider detail.
         */
        detailRenderer: null,

        /**
         * Class responsible for rendering map view results.
         */
        mapRenderer: null,

        /**
         * Pagination handler.
         */
        pagination: null,

        /**
         * Search criteria handler.
         */
        searching: null,

        /**
         * Sorting handler.
         */
        sorting: null,

        /**
         * Loading overlay handlers.
         */
        resultsLoading: null,
        detailLoading: null,

        /**
         * data loader object.
         */
        dmClient: null,

        /**
         * Data localization handler.
         */
        localizer: null,

        /**
         * Data localization handler.
         */
        categories: null,

        /**
         * Page title
         */
        originalTitle: null,

        /**
         * Discount finder constructor.
         * @param wrapper Element wrapping discount finder HTML structure.
         * @param config Configuration attributes.
         */
        constructor: function(wrapper, config) {
            if (!config.ajaxUrl) {
                throw new Error("Configuration has to contain ajaxUrl for wordpress ajax calls.");
            }
            if (!config.locale) {
                throw new Error("Configuration has to contain current locale.");
            }
            if (!config.categories) {
                throw new Error("Mapo images has to be set.");
            }
            // Set configuration variables.
            this.ajaxUrl = config.ajaxUrl;
            this.categories = config.categories;
            this.localizer = new DmLocalizer(config.locale);
            // FIXME: @deprecated Tahle metoda je podivná, navíc nevím proč je na globálním kontextu.
            window.dmGetProviderUrl = function(provider, cardType) {
                if (!window.location.origin) {
                    window.location.origin = window.location.protocol + "//" + window.location.hostname + (window.location.port ? ':' + window.location.port: '');
                }
                var params = this.parseHttpQuery(window.location.search.substr(1));
                for (var i in params) {
                    if (i.substr(0, 2) == 'dm') {
                        delete params[i];
                    }
                }
                if (cardType) {
                    params.dmCardType = cardType;
                }
                var matches = window.location.pathname.match(/(.*\/)[^\/]+\/[0-9]+/);
                var path = matches ? matches[1] : window.location.pathname;
                var slug = this.localizer.localizedProviderValue(provider, 'slug')
                var result = window.location.origin + path + slug + '/' + provider.id;
                if ($.param(params)) {
                    result += '?' + $.param(params);
                }
                return result;
            }.bind(this);

            // FIXME: @deprecated Tahle metoda je podivná, navíc nevím proč je na globálním kontextu.
            window.dmGetDiscountSearchUrl = function(params) {
                if (!window.location.origin) {
                    window.location.origin = window.location.protocol + "//" + window.location.hostname + (window.location.port ? ':' + window.location.port: '');
                }
                var matches = window.location.pathname.match(/(.*\/)[^\/]+\/[0-9]+/);
                var path = matches ? matches[1] : window.location.pathname;
                return window.location.origin + path + '?' + $.param(params);
            };

            this.dmClient = new DmClient(this.ajaxUrl);
            this.wrapper = wrapper;
            this.resultsWrapper = $('[data-dm-target="results"]', this.wrapper);
            this.mapResultsWrapper = $('[data-dm-target="map-results"]', this.wrapper);
            this.contentWrapper = $('[data-dm-target="content-wrapper"]', this.wrapper);
            // Initializae handlers.
            this.searching = new DmSearching(wrapper, this.ajaxUrl, this.localizer);
            this.pagination = new DmPagination(null);
            this.sorting = new DmSorting(wrapper, this.localizer);
            // Initialize overlay handler.
            this.resultsLoading = new DmLoadingOverlay(this.resultsWrapper);
            // Initialize application.
            this.init(config);
            // Initialize title
            this.originalTitle = document.title;
        },

        /**
         * Initialize the discount finder.
         * @param config Application configuration object.
         */
        init: function(config) {
            // Attach all listeners necessary for the
            this.attachListeners();
            this.restoreState();
        },

        /**
         * Attach all necessary listeners to handle the application events.
         */
        attachListeners: function() {
            var that = this;
            // Display of the results is changed.
            $('[data-dm-target="display-switch"]', this.wrapper).on('click', function(event) {
                that.handleDisplaySwitch(event);
                return false;
            });
            this.wrapper.on('click', function(event) {
                if ($(event.target).data('dm-provider-id')) {
                    if (event.button != 0) {
                        return true;
                    } else {
                        that.providerId = $(event.target).data('dm-provider-id');
                        that.showProviderDetail(that.providerId, that.searching.cardType, true);
                        that.scrollToTop();
                        return false;
                    }
                }
            });
            this.wrapper.on(
                'click',
                '[data-dm-reset-filters]',
                function (e) {
                    that.searching.resetState();
                    that.storeState();
                    that.search();
                    return false;
                }
            );
            this.wrapper.on('click', '.dm-tag', function(event) {
                that.pagination.reset();
                that.searching.resetState();
                that.searching.setState( { text: $(this).data('dm-tag') } );
                that.providerId = null;
                that.storeState();
                that.search();
                return false;
            });
            this.wrapper.on('click', 'a.dm-social-network', function() {
                window.open($(this).attr('href'), '_blank', "width=530, height=500, scrollbars=0");
                return false;
            });
            // Any search params are changed.
            this.searching.onChange = function() {
                // New search. Reset pagination.
                that.pagination.reset();
                that.search();
                that.storeState();
            }
            // Sort key or sort order is chnged.
            this.sorting.onChange = function() {
                // New search. Reset pagination.
                that.pagination.reset();
                that.search();
                that.storeState();
            }
            // Page has been chaged.
            this.pagination.onPageChange = function() {
                that.search();
                that.storeState();
                that.scrollToTop();
            }
            // More results button was clicked.
            this.pagination.onMoreResults = function(offset, limit) {
                that.append(offset, limit);
                that.storeState();
            }
            // Back button was clicked.
            $(window).on('popstate', function(event) {
                // This event happens on page load in chrome, but in other browsers not.
                // It causes that the restore state is called twice, but there is no simple solution for this.
                that.restoreState();
                // Set original title
                document.title = that.originalTitle;
                // Remove description
                $("[name=description]").remove();
            });
            $('#dm-scroll-to-top').on('click', function() {
                that.scrollToTop();
                return false;
            });
        },

        /**
         * Scrool to the top of the plugin.
         */
        scrollToTop: function () {
            $('html, body').animate({ scrollTop: jQuery('#dm-discount-finder').offset().top - 150 }, 300);
        },

        /**
         * Handle the display change swithch.
         * @param event Event causing the change.
         */
        handleDisplaySwitch: function(event) {
            if ($(event.target).data("dm-display")) {
                this.switchDisplay($(event.target).data("dm-display"));
                this.storeState();
            }
            // TODO: Set correct css classes.
            event.stopPropagation()
        },

        /**
         * Restore state of te application based on the query params.
         * It might be good to accept some default values as parameter to be able to replace the default values.
         */
        restoreState: function() {
            // Split each query pair and remove prefix.
            var queryParams = this.parseHttpQuery(window.location.search.substring(1), this.queryParamsPrefix);
            var matches = window.location.pathname.match(/\/[^\/]+\/[^\/]+\/([0-9]+)\/?$/);
            var providerId = matches === null ? null : matches[1];
            // Set states of each componenet based on query.
            this.pagination.setState(queryParams);
            this.sorting.setState(queryParams);
            this.searching.setState(queryParams);
            var display = queryParams.display ? queryParams.display : 'images';
            this.providerId = queryParams.providerId ? queryParams.providerId : providerId;
            if (this.providerId != null) {
                this.showProviderDetail(this.providerId, queryParams.cardType, false);
            } else {
                if (this.detailRenderer) {
                    this.detailRenderer.destroy();
                }
                this.switchDisplay(display);
            }
        },

        /**
         * Serialize context to the browser query and alter browser history.
         */
        storeState: function() {
            // XXX: it might be good to check whether the state has changed.
            var queryParams = {};
            var searchCriteria = this.searching.getSearchParams();
            var pagingParams = this.pagination.getPagingParams();
            var sortParams = this.sorting.getSortParams();
            // Serialize state to the paging params.
            for (var key in searchCriteria) {
                var queryKey = this.queryParamsPrefix + key.charAt(0).toUpperCase() + key.slice(1)
                queryParams[queryKey] = searchCriteria[key];
            }
            for (var key in pagingParams) {
                var queryKey = this.queryParamsPrefix + key.charAt(0).toUpperCase() + key.slice(1)
                queryParams[queryKey] = pagingParams[key];
            }
            for (var key in sortParams) {
                var queryKey = this.queryParamsPrefix + key.charAt(0).toUpperCase() + key.slice(1)
                queryParams[queryKey] = sortParams[key];
            }
            queryParams[this.queryParamsPrefix + 'Display'] = this.display;
            queryParams[this.queryParamsPrefix + 'ProviderId'] = this.providerId;
            // Push new state to the browser history.
            if (history.pushState) {
                var oldQueryParams = this.parseHttpQuery(window.location.search.substring(1), null);
                var newQueryParams = $.extend(oldQueryParams, queryParams);
                for (var key in newQueryParams) {
                    // Remove empty or undefined discount manager parameters.
                    if (queryParams.hasOwnProperty(key) && !newQueryParams[key]) {
                        delete newQueryParams[key];
                    }
                }
                var newurl;
                if (this.providerId) {
                    newurl = window.dmGetProviderUrl(this.provider, queryParams.dmCardType);
                } else {
                    newurl = window.dmGetDiscountSearchUrl(newQueryParams);
                }
                window.history.pushState({ url: newurl, tag: 'dmState' }, '', newurl);
            }
        },

        /**
         * Parse HTTP query string and return it's components as an object with values.
         * If prefix is specified, only prefixed parameters will be parsed and prefix will be removad from the result.
         * @param string query Query string. Can be null.
         * @param string prefix Camel case prefix of the query parameter. Can be null.
         */
        parseHttpQuery: function(query, prefix) {
            var queryParams = {}
            if (!query) { // If nothing is suppliend, return empty object.
                return queryParams;
            }
            var vars = query.split('&');
            for (var i = 0; i < vars.length; i++) {
                var pair = vars[i].split('=');
                if (prefix) {
                    if (pair[0].indexOf(prefix) == 0) { // Parse only parameters with prefix.
                        var key = decodeURIComponent(pair[0]).slice(prefix.length);
                        key = key.charAt(0).toLowerCase() + key.slice(1);
                        queryParams[key] = decodeURIComponent(pair[1].replace(new RegExp('\\+', 'g'), '%20'));
                    }
                } else {
                    queryParams[decodeURIComponent(pair[0])] = decodeURIComponent(pair[1].replace(new RegExp('\\+', 'g'), '%20'));
                }
            }
            return queryParams;
        },

        /**
         * Instead of replacing the results appen results to the end fo the list.
         * Valid only for list and images view.
         */
        append: function(offset, limit) {
            // Show results as list or images.
            if (this.display == 'list' || this.display == 'images') {
                $('[data-dm-target="pagination-load-more"] .dm-icon-refresh').addClass("dm-rotate");
                // Choose corect renderer.
                var renderer = this.display == 'list' ? this.listRenderer : this.imagesRenderer;
                // Paging params are from the parameters.
                var pagingParams = {offset: offset, limit: limit};
                // Get search criteria.
                var serachCriteria = this.searching.getSearchCriteria();
                // Get paging and sorting attributes.
                var sortParams = this.sorting.getSortParams();
                // Combine the variables to make parameters for the request.
                var requestParams = $.extend(serachCriteria, pagingParams, sortParams);
                this.dmClient.getProviders(requestParams, function(data) {
                    renderer.append(data['items']);
                    $('[data-dm-target="pagination-load-more"] .dm-icon-refresh').removeClass("dm-rotate");
                });
            } else {
                throw new Error("Appending not implemented for diplay type: '" + this.display + "'.");
            }
        },

        /**
         * Perform search of tehe results based on the pplicaiton context.
         */
        search: function() {
            var that = this;
            // Destroy detail if detail is shown.
            if (this.detailRenderer) {
                this.detailRenderer.destroy();
                this.providerId = null;
                that.switchDisplay(this.display ? this.display : 'images'); // Ugly fix of missing display option.
            }
            if (this.display == null) {
                this.display = 'map';
            }

            // Search criteria are always necessary.
            var searchCriteria = this.searching.getSearchCriteria();
            // Show results as list or images.
            if (this.display == 'list' || this.display == 'images') {
                // Show loading icon.
                this.resultsLoading.show();
                // Choose corect renderer.
                var renderer = this.display == 'list' ? this.listRenderer : this.imagesRenderer;
                // Get paging and sorting attributes.
                var pagingParams = this.pagination.getPagingParams();
                var sortParams = this.sorting.getSortParams();
                // Combine the variables to make parameters for the request.
                var requestParams = $.extend(searchCriteria, pagingParams, sortParams, {facets: ['cardTypes', 'categories']});
                this.dmClient.getProviders(requestParams, function(data) {
                    that.resultsLoading.hide(); // Hide loading icon.
                    if (data) {
                        that.updateFacets(data['facets']); // Update facets.
                    }
                    if (data) {
                        if ((data['count'] > 0)) {
                            renderer.replace(data['items'], that.searching.cardType); // Replace the content with the new ites.
                            that.pagination.setCount(data['count']); // Update pagination.
                            that.pagination.renderPages();
                        } else {
                            that.nothingFound();
                        }
                    }

                });
            } else if (this.display == 'map') {
                if (!this.mapRenderer.isBoundingBox()) {
                    // no bounding box yet, map not initialized
                    return;
                }
                // Search criteria are always necessary.
                var searchCriteria = this.searching.getSearchCriteria();
                var requestParams = $.extend(
                    searchCriteria,
                    {
                        boundingBox: this.mapRenderer.getBoundingBox(),
                        limit: 100,
                        facets: ['cardTypes', 'categories']
                    }
                );
                this.dmClient.getBranches(requestParams, (function(data) {
                    this.resultsLoading.hide();
                    if (data) {
                        this.updateFacets(data['facets']); // Update facets.
                        this.mapRenderer.replace.bind(this.mapRenderer)(
                            data['items'],
                            this.searching.categoryId,
                            this.searching.cardType,
                            this.searching.text
                        );
                    }
                }).bind(this));
            } else {
                throw new Error("Unknown display option: '" + this.display + "'.");
            }
        },

        /**
         * Display message that nothing has been found.
         */
        nothingFound: function() {
            this.resultsWrapper.empty();
            this.resultsWrapper.append(
                $('<div />').
                        addClass('dm-nothing-found').
                        append(
                            $('<strong />').
                                text(discount_search.nothing_found)
                        ).
                        append($('<br />')).
                        append(
                            $('<span />').
                                text(discount_search.try_something_else + ' ')
                        ).
                        append(
                            $('<a />').
                                text(discount_search.all_discounts).
                                attr('data-dm-reset-filters', '1').
                                attr('href', '#')
                        )
            );
        },

        /**
         * Update values of the facets.
         */
        updateFacets: function(facets) {
            $('[data-dm-target="filter-card-types-card-type"]').each(function (index, element) {
                var cardType = $(element).data('dm-card-type').toLowerCase();
                var countElement = $(element).children('[data-dm-target="filter-card-types-card-type-count"]');
                if ((facets['cardTypes'][cardType] && countElement.html() != ('(' + facets['cardTypes'][cardType] + ')')) ||
                        (!facets['cardTypes'][cardType] && countElement.html() != '(0)')) {
                    countElement.hide();
                    countElement.html('(' + (facets['cardTypes'][cardType] ? facets['cardTypes'][cardType] : '0') + ')');
                    countElement.fadeIn(300);
                }
            });
            $('[data-dm-target="filter-categories-category"]').each(function (index, element) {
                var categoryId = $(element).data('dm-category-id');
                var countElement = $(element).children('[data-dm-target="filter-categories-category-count"]');
                if ((facets['categories'][categoryId] && countElement.html() != ('(' + facets['categories'][categoryId] + ')')) ||
                        (!facets['categories'][categoryId] && countElement.html() != '(0)')) {
                    countElement.hide();
                    countElement.html('(' + (facets['categories'][categoryId] ? facets['categories'][categoryId] : '0') + ')');
                    countElement.fadeIn(300);
                }
            });
        },

        /**
         * Switch current display of the results. Initialize the renderers and upadate application context.
         */
        switchDisplay: function(display) {
            if (this.display == display) {
                return;
            }
            // Initialize renderers and update apropreate context variables.
            switch (display) {
                case 'list':
                    if (this.listRenderer == null) {
                        this.listRenderer = new DmListRenderer(this.resultsWrapper, this.localizer);
                    }
                    $('.dm-pagination-pages').fadeIn(300);
                    $('.dm-show-more-wrapper').fadeIn(300);
                    $('.dm-sorting-switch').fadeIn(300);
                    this.resultsWrapper.show();
                    this.mapResultsWrapper.hide();
                    this.display = 'list';
                    this.pagination.setPageSize(this.pageSize.list);
                    break;
                case 'images':
                    if (this.imagesRenderer == null) {
                        this.imagesRenderer = new DmImagesRenderer(this.resultsWrapper, this.localizer, this.categories);
                    }
                    this.display = 'images';
                    $('.dm-pagination-pages').fadeIn(300);
                    $('.dm-show-more-wrapper').fadeIn(300);
                    $('.dm-sorting-switch').fadeIn(300);
                    this.resultsWrapper.show();
                    this.mapResultsWrapper.hide();
                    this.pagination.setPageSize(this.pageSize.images);
                    break;
                case 'map':
                    if (this.mapRenderer == null) {
                        if (discount_search.map_provider == 'google') {
                            this.mapRenderer = new DmMapRenderer(this.mapResultsWrapper, this.localizer, this.categories, 'list');
                            this.mapRenderer.onChange = function() {
                                this.search();
                            }.bind(this);
                        } else {
                            this.mapRenderer = new DmMapRendererStreet(this.mapResultsWrapper, this.localizer, this.categories, 'list');
                            this.mapRenderer.onChange = function() {
                                this.search();
                            }.bind(this);
                        }

                    }
                    this.resultsWrapper.hide();
                    this.mapRenderer.takeControl();
                    $('.dm-pagination-pages').fadeOut(300);
                    $('.dm-show-more-wrapper').fadeOut(300);
                    $('.dm-sorting-switch').fadeOut(300);
                    this.display = 'map';
                    break;
                default:
                    throw new Error("Unknown display option: '" + display + "'.");
            }
            $('[data-dm-target="display-switch"] a').removeClass('dm-btn-active');
            $('[data-dm-target="display-switch"] a[data-dm-display="' + display + '"]').addClass('dm-btn-active');
            this.pagination.reset();
            // Perform search.
            this.search();

        },

        /**
         * Show detail of the provider with given identifier.
         * @param providerId Provider identifier.
         */
        showProviderDetail: function(providerId, cardType, storeState) {
            var that = this;
            this.providerId = providerId;
            // Show loading icon.
            if (this.detailLoading === null) {
                this.detailLoading = new DmLoadingOverlay(this.contentWrapper);
            }
            if (this.dmClient.ajaxUrl != 'not initialized') {
                that.detailLoading.show();
            }

            this.dmClient.viewProvider({providerId: providerId});

            this.dmClient.getProvider({providerId: providerId}, function (data) {
                that.provider = data;
                that.detailLoading.hide();
                if (that.detailRenderer === null) {
                    that.detailRenderer = new DmDetailRenderer(that.contentWrapper, that.localizer,
                            that.dmClient, that.categories, that.searching.getAvailableCardTypes());
                }
                that.detailRenderer.render(data, cardType);
                that.scrollToTop();
                if (storeState) {
                    that.storeState();
                }
            });
        }


    });

    /**
     * Class responsible for handling loading overlay of the images.
     */
    window.DmLoadingOverlay = function(element) {
        this.constructor.call(this, element);
    }
    $.extend(DmLoadingOverlay.prototype, {

        element: null,
        overlay: null,
        /**
         * Construct overlay handler.
         * @param element JQuery element where overlay should be shown.
         */
        constructor: function(element) {
            // TODO: missing styling.
            this.element = $(element);
            this.element.css('position', 'relative'); // Should be in css.
        },

        /**
         * Show overlay on the element.
         */
        show: function() {
            if (this.overlay) { // Prevent showing multiple instances of the overlay.
                return;
            }
            this.overlay = $('<div>').
                css('width', this.element.innerWidth()).
                css('height', this.element.innerHeight()).
                css('top', 0).
                css('left', 0).
                css('position', 'absolute');
            this.overlay.addClass('dm-loading-overlay');
            this.element.append(this.overlay);
        },

        /**
         * Hide overlay on the element.
         */
        hide: function() {
            if (!this.overlay) { // We only want to close it when it's shown.
                return;
            }
            this.overlay.remove();
            delete this.overlay;
        }

    });


    /**
     * Class responsible for rendering list of the providers.
     */
    window.DmListRenderer = function(element, localizer) {
        this.constructor.call(this, element, localizer);
    }
    $.extend(DmListRenderer.prototype, {

        /**
         * Element where the list should be rendered.
         */
        element: null,

        /**
         * Data localizer.
         */
        localizer: null,

        /**
         * Construct the renderer object.
         */
        constructor: function(element, localizer) {
            this.element = $(element);
            this.localizer = localizer;
        },

        /**
         * Append providers to the given list.
         */
        append: function(providers) {
            this.render(providers);
        },

        /**
         * Replace list with new set of providers.
         */
        replace: function(providers, cardType) {
            this.element.html("");
            this.render(providers, cardType);
        },

        /**
         * Render the provider list to tasrget element.
         */
        render: function(providers, cardType) {
            for (var i = 0; i < providers.length; i++) {
                var item = $(this.renderItem(providers[i], cardType));
                item.hide();
                this.element.append(item);
                item.delay( i * 100 - 50 ).slideDown({ duration : 100 });
            }
        },

        /**
         * Render item of the list for given provider.
         */
        renderItem: function(provider, cardType) {
            var content = $('<div />').
                    addClass('dm-provider-list-item-content');
            var header = $('<h2 />').
                    text(this.localizer.localizedProviderValue(provider, 'name')).
                    appendTo(content);
            for (var i = 1; i - 0.5 <= provider.providerStats.averageRating; i++) {
                var star = $('<span />').addClass('dm-rating');
                if (i <= provider.providerStats.averageRating) {
                    star.addClass('fa fa-star');
                } else if (i - 0.5 <= provider.providerStats.averageRating) {
                    star.addClass('fa fa-star-half-full');
                }
                star.appendTo(header);
            }
            var imageWrapper =  $('<div />').
                    data('dm-provider-id', provider.id).
                    addClass('dm-provider-list-item-logo');
            var logoUrl = null;
            $.each(provider.logo, function(ix, image) {
                if (image.type == 'SMALL') {
                    logoUrl = image.url;
                    return false;
                }
            });
            if (logoUrl === null) {
                logoUrl = provider.logoUrl;
            }
            imageWrapper.append(
                    $('<a />').
                            attr('href', window.dmGetProviderUrl(provider, cardType)).
                            css('width', 100).
                            css('height', 100).
                            css('display', 'inline-block').
                            append(
                                    $('<img />').
                                            prop('src', logoUrl).
                                            on('load', function (event) {
                                                $(event.target).parent().parent().css('background', 'none');
                                                $(event.target).parent().removeAttr('style');
                                            }).
                                            data('dm-provider-id', provider.id)
                            )
            );
            var response = $('<div />').
                    addClass('dm-provider-list-item').
                    append(imageWrapper).
                    append(content);
            var categories = {};
            var infolineElement = $('<div />').addClass('dm-provider-list-item-discount-infoline').appendTo(content);

            content.append(
                    $('<div />').
                            addClass('dm-provider-list-item-discount').
                            append($('<span />').text(this.localizer.localizedProviderValue(provider, 'shortDescription')))
            );

            content.append(
                    $('<div />').
                            addClass('dm-provider-user-content-counts').
                            append($('<span />').text(provider.providerStats.commentCount)).
                            append($('<span />').addClass('fa fa-comments-o')).
                            append($('<span />').text(provider.providerStats.photoCount + (provider.imageUrl ? 1 : 0))).
                            append($('<span />').addClass('fa fa-picture-o '))
            );

            if (provider.discounts) {
                for (var i = 0; i < provider.discounts.length; i++) {
                    var currentDate = new Date(new Date().toISOString().slice(0, 10)).getTime();
                    var validSince = new Date(new Date(provider.discounts[i].validSince).toISOString().slice(0, 10)).getTime();
                    var validUntil = provider.discounts[i].validUntil ? new Date(new Date(provider.discounts[i].validUntil).toISOString().slice(0, 10)).getTime() : null;
                    if (provider.discounts[i].active && validSince <= currentDate && (!provider.discounts[i].validUntil || validUntil >= currentDate)) {
                        for(var j = 0; j < provider.discounts[i].categories.length; j++) {
                            categories[provider.discounts[i].categories[j].name] = provider.discounts[i].categories[j];
                        }
                    }
                }
            }
            var categoriesText = '';
            for (var category in categories) {
                if (categoriesText) {
                    categoriesText += ', ';
                }
                categoriesText += $('<div />').text(this.localizer.localizedCategoryValue(categories[category], 'name')).html();
            }
            infolineElement.append(categoriesText);
            infolineElement.append($('<span>').addClass('dm-line-separator').text('|'));
            if (provider.branchCount == 0) {
                infolineElement.append(
                        $('<span>').addClass('dm-provider-images-item-location').text(discount_search.provider_online)
                );
            } else if (provider.branchCount == 1) {
                var address = provider.branch.address.address1 + ', ' + provider.branch.address.city;
                infolineElement.append(
                        $('<span></span>').addClass('dm-provider-images-item-location').text(address)
                );
            } else {
                infolineElement.append(
                        $('<span></span>').addClass('dm-provider-images-item-location').text(discount_search.list_images_moreBranches)
                );
            }
            content.append(
                    $('<div />').
                            addClass('dm-provider-list-item-discount-actionline').
                            append(
                                    $('<a />').
                                            attr('href', window.dmGetProviderUrl(provider, cardType)).
                                            append(
                                                    $('<button />').
                                                            addClass('dm-provider-list-item-discount-button').
                                                            addClass('dm-btn').
                                                            addClass('dm-btn-active').
                                                            data('dm-provider-id', provider.id).
                                                            text(this.localizer.message('showProvider'))
                                            )
                            )
            );
            return response;
        }

    });

    /**
     * Class responsible for rendering images of the providers.
     */
    window.DmImagesRenderer = function(element, localizer, categories) {
        this.constructor.call(this, element, localizer, categories);
    }
    $.extend(DmImagesRenderer.prototype, {

        /**
         * Element where the list should be rendered.
         */
        element: null,

        /**
         * Categories handler.
         */
        categories: null,

        /**
         * Construct the renderer object.
         */
        constructor: function(element, localizer, categories) {
            this.element = $(element);
            this.localizer = localizer;
            this.categories = categories;
        },

        /**
         * Append providers to the given list.
         */
        append: function(providers) {
            this.render(providers);
        },

        /**
         * Replace list with new set of providers.
         */
        replace: function(providers, cardType) {
            this.element.html("");
            this.render(providers, cardType);
        },

        /**
         * Render the provider list to tasrget element.
         */
        render: function(providers, cardType) {
            for (var i = 0; i < providers.length; i++) {
                var item = $(this.renderItem(providers[i], cardType));
                item.hide();
                this.element.append(item);
                item.delay( i * 100 - 50 ).show({ duration : 100 });
            }
        },

        getProviderImageUrl: function(provider) {
            var imageUrl = null;
            $.each(provider.image, function(ix, image) {
                if (image.type == 'NORMAL') {
                    imageUrl = image.url;
                    return false;
                }
            });
            if (imageUrl) {
                return imageUrl;
            }
            if (provider.imageUrl) {
                return provider.imageUrl;
            }
            for (var i = 0; i < provider.discounts.length; i++) {
                for (var j = 0; j < provider.discounts[i].categories.length; j++) {
                    if (this.categories[provider.discounts[i].categories[j].id]['imageUrl']) {
                        return this.categories[provider.discounts[i].categories[j].id]['imageUrl'];
                    }
                };
            };
            return discount_search.default_provider_img;
        },

        /**
         * Render item of the list for given provider.
         */
        renderItem: function(provider, cardType) {
            var response = $('<div></div>') .
                addClass('dm-provider-images-item');

            var imgWrapper = $('<div></div>') .
                addClass('dm-provider-images-item-logo') .
                appendTo(response);
            var imgA = $('<a />').
                    data('dm-provider-id', provider.id).
                    attr('href', window.dmGetProviderUrl(provider, cardType)).
                    appendTo(imgWrapper);
            var rating = $('<div />').addClass('dm-rating-wrapper').appendTo(imgWrapper);
            for (var i = 1; i <= 5; i++) {
                var star = $('<span />').addClass('dm-rating');
                if (i <= provider.providerStats.averageRating) {
                    star.addClass('fa fa-star').addClass('dm-rating-active');
                } else if (i - 0.5 <= provider.providerStats.averageRating) {
                    star.addClass('fa fa-star-half-full').addClass('dm-rating-active');
                } else {
                    star.addClass('fa fa-star').addClass('dm-rating-inactive');
                }
                star.appendTo(rating);
            }

            var imgUrl = this.getProviderImageUrl(provider);
            if (imgUrl != '') {
                var img = $('<img />').
                    data('dm-provider-id', provider.id).
                    prop('src', imgUrl).
                    appendTo(imgA);
            }

            var content = $('<div></div>').
                addClass('dm-provider-images-item-content').
                appendTo(response);

            response.append(
                    $('<div />').
                            addClass('dm-provider-user-content-counts').
                            append($('<span />').text(provider.providerStats.commentCount)).
                            append($('<span />').addClass('fa fa-comments-o')).
                            append($('<span />').text(provider.providerStats.photoCount + (provider.imageUrl ? 1 : 0))).
                            append($('<span />').addClass('fa fa-picture-o '))
            );

            var header = $('<h2></h2>').
                    text(this.localizer.localizedProviderValue(provider, 'name')).
                    appendTo(content);

            var categories = $('<div></div>').appendTo(content);
            categories.addClass('dm-provider-images-item-categories');

            var description = $('<div></div>').
                addClass('dm-provider-images-item-discount').
                appendTo(content);

            $('<span></span>').
                    text(this.localizer.localizedProviderValue(provider, 'shortDescription')).
                    appendTo(description);

            var categoryList = {};
            if (provider.discounts) {
                for (var i = 0; i < provider.discounts.length; i++) {
                    var currentDate = new Date(new Date().toISOString().slice(0, 10)).getTime();
                    var validSince = new Date(new Date(provider.discounts[i].validSince).toISOString().slice(0, 10)).getTime();
                    var validUntil = provider.discounts[i].validUntil ? new Date(new Date(provider.discounts[i].validUntil).toISOString().slice(0, 10)).getTime() : null;
                    if (provider.discounts[i].active && validSince <= currentDate && (!provider.discounts[i].validUntil || validUntil >= currentDate)) {
                        for(var j = 0; j < provider.discounts[i].categories.length; j++) {
                            categoryList[provider.discounts[i].categories[j].name] = provider.discounts[i].categories[j];
                        }
                    }
                }
            }

            var categoriesText = '';
            for (var category in categoryList) {
                if (categoriesText) {
                    categoriesText += ', ';
                }
                categoriesText += $('<div />').text(this.localizer.localizedCategoryValue(categoryList[category], 'name')).html();
            }
            categories.append(categoriesText);
            categories.append($('<span>').addClass('dm-line-separator').text('|'));
            if (provider.branchCount == 0) {
                $('<span></span>').
                    addClass('dm-provider-images-item-location').
                    text(discount_search.provider_online).
                    appendTo(categories);
            } else if (provider.branchCount == 1) {
                var address =
                    provider.branch.address.address1 +
                    ', ' +
                    provider.branch.address.city;
                $('<span></span>').
                    addClass('dm-provider-images-item-location').
                    text(address).
                    appendTo(categories);
            } else {
                $('<span></span>').
                    addClass('dm-provider-images-item-location').
                    text(discount_search.list_images_moreBranches).
                    appendTo(categories);
            }

            var viewDiscountBtn = $('<button></button>').
                addClass('dm-btn').
                addClass('dm-btn-active').
                addClass('dm-provider-images-item-discount-button').
                data('dm-provider-id', provider.id).
                text(discount_search.list_images_viewDiscount).
                appendTo(
                        $('<a />').
                                attr('href', window.dmGetProviderUrl(provider, cardType)).
                                appendTo(response)
                );

            return response;
        }

    });

    /**
     * Class responsible for rendering map of the providers.
     */
    window.DmMapRendererStreet = function(element, localizer, categories, type) {
        this.constructor.call(this, element, localizer, categories, type);
    }
    $.extend(DmMapRendererStreet.prototype, {
        /**
         * Construct the renderer object.
         */
        constructor: function(element, localizer, categories, type) {
            this.element = $(element);
            this.localizer = localizer;
            this.categories = categories;

            this.map = null;
            this.mapElement = null;
            this.infowindowConfiguration = type;
            this.markers = {};

            this.mapElement = $('<div></div>').addClass('dm-provider-map');
            // this.mapElement.style = 'height:500px;width:500px;';
            this.element.append(this.mapElement);

            if (this.infowindowConfiguration == 'list') {
                this.createLegend();
            }

            // Create Leaflet map on map element.
            this.map = L.map(this.mapElement.get(0), {
                center: [discount_search.initial_lat, discount_search.initial_lon],
                zoom: +discount_search.initial_zoom,
                maxZoom: 17
            });
            var that = this;
            this.map.on("zoomend", function (e) { that.onChange(); });
            this.map.on("moveend", function (e) { that.onChange(); });


            // Add OSM tile leayer to the Leaflet map.
            L.tileLayer('https://{s}.tile.osm.org/{z}/{x}/{y}.png', {
                attribution: '&copy; <a href="https://osm.org/copyright">OpenStreetMap</a> contributors'
            }).addTo(this.map);
        },

        onChange: function() {
        },

        takeControl: function() {
            this.element.show();
            this.map.invalidateSize();
        },
        createLegend: function() {
            var legend = $('<div></div>') .
            addClass('dm-map-legend').
            appendTo(this.element);
            $('<h2></h2>') .
            text(discount_search.list_map_legend) .
            appendTo(legend);
            var categoriesWrapper = $('<div></div>') .
            addClass('dm-map-legend-categories') .
            addClass('clearfix') .
            appendTo(legend);

            var categories = ['travel', 'transportation', 'eshop', 'festival', 'foodAndDrink', 'shopping', 'culture', 'skiResort', 'sport', 'accommodation', 'education', 'funAndRelax', 'other'];
            for (var i = 0; i < categories.length; i++) {
                var categoryWrapper = $('<div></div>').addClass('dm-map-legend-categories-category').appendTo(categoriesWrapper);
                var imgWrapper = $('<dd></dd>').appendTo(categoryWrapper);
                var img = $('<span></span>') .
                    addClass('dm-category-icon') .
                    addClass('dm-category-icon-' + categories[i]).
                    appendTo(imgWrapper);
                var text = $('<dt></dt>').
                    text(discount_search['category_' + categories[i]]).
                    appendTo(categoryWrapper);
            };
        },

        isBoundingBox: function() {
            var bounds = this.map.getBounds();
            if (bounds == null || typeof bounds == 'undefined') {
                return false;
            } else {
                return true;
            }
        },

        getBoundingBox: function() {
            var bounds = this.map.getBounds();
            return bounds.getNorthEast().lat + "," + bounds.getSouthWest().lng +
                ":" +
                bounds.getSouthWest().lat + "," + bounds.getNorthEast().lng;
        },


        /*
         * Updates map with new providers nad stuff.
         */
        replace: function(items, categoryId, cardType, searchText) {
            if (this.categoryId != categoryId || this.cardType != cardType || this.searchText != searchText) {
                this.deleteAllMarkers.bind(this)();
            }
            this.showMarkers.bind(this)(items);
            this.categoryId = categoryId;
            this.cardType = cardType;
            this.searchText = searchText;
        },

        deleteAllMarkers: function() {
            for (var key in this.markers) {
                this.map.removeLayer(this.markers[key]);
            };
            this.markers = {};
        },

        showMarkers: function(items) {
            var fg = L.featureGroup().addTo(this.map);
            var bounds = new L.LatLngBounds();
            for (var i = items.length - 1; i >= 0; --i) {
                if (!(items[i].id in this.markers)) {
                    if (!items[i].location || !items[i].discounts || items[i].discounts.length == 0 || items[i].location.active) {
                        continue;
                    }
                    var myIcon = L.icon({
                        iconUrl: discount_search['plugins_url'] + '/resources/alive-dm/img/' + this.getCategoryTypeForDiscounts(items[i].discounts) + '.png',
                        iconSize: [32, 37],
                        popupAnchor: [0,0],
                    });

                    var marker = L.marker(L.latLng(items[i].location.lat, items[i].location.lon), {icon: myIcon}).addTo(fg);
                    bounds.extend(marker.getLatLng());
                    var text = this.getText(items[i], this.infowindowConfiguration,  marker, this.map, this.localizer, this.cardType);
                    marker.bindPopup(text.html(), {className: 'custom-popup'});
                    marker.addTo(this.map);

                    this.markers[items[i].id] = marker;
                }
            };
            if (this.infowindowConfiguration == 'detail') {
               var map = this.map;
                setTimeout(function () {
                    map.fitBounds(fg.getBounds());
                }, 1000);
            }
        },

        getCategoryTypeForDiscounts: function(discounts) {
            var categories = [];
            for (var i = 0; i < discounts.length; i++) {
                for (var j = 0; j < discounts[i].categories.length; j++) {
                    categories.push(discounts[i].categories[j].id);
                };
            };
            var pickedCategoryId = categories.sort(function(a, b) {return a-b})[0];
            if (this.categories[pickedCategoryId]) {
                return this.categories[pickedCategoryId]['icon'];
            } else {
                return 'other';
            }
        },

        getText: function (branch, configuration, marker, map, localizer, cardType) {
            this.branch_ = branch;
            this.configuration_ = configuration;
            this.marker_ = marker;
            this.map_ = map;
            this.localizer_ = localizer;
            this.cardType = cardType;

            var content = $('<div>');

            var name = $('<h1>').
            text(this.localizer_.localizedBranchValue(this.branch_, 'name')).
            appendTo(content);

            var discounts = $('<div style="margin-top: 15px;"></div>').
            addClass('info-discounts').
            appendTo(content);

            var wrapper;

            if (this.configuration_ == 'detail') {
                wrapper = $('<ul />').appendTo(discounts);
            } else if (this.configuration_ == 'list') {
                wrapper = discounts;
            }
            for (var i = 0; i < this.branch_.discounts.length; i++) {
                var currentDate = new Date().getTime();
                var validSince = new Date(this.branch_.discounts[i].validSince).getTime();
                var validUntil = new Date(this.branch_.discounts[i].validUntil).getTime();

                if (!this.branch_.discounts[i].active || (validSince > currentDate || validUntil < currentDate)) {
                    continue;
                }
                var discount;
                if (this.configuration_ == 'detail') {
                    discount = $('<li />');
                } else if (this.configuration_ == 'list') {
                    discount = $('<span /><br />').
                    addClass('infowindow-discount');
                }
                discount.
                text(this.localizer_.localizedDiscountValue(this.branch_.discounts[i], 'name')).
                appendTo(wrapper);
            };

            var address = $('<div></div>').
            addClass('infowindow-address').
            html('<strong>' + discount_search.infowindow_address + '</strong><br />').
            appendTo(content);
            var addr = this.branch_.address;
            address.append(
                $('<span></span>').
                append($('<span></span>').text(addr.address1 + (addr.address2 ? ', ' + addr.address2 : ''))).
                append($('<br />')).
                append($('<span></span>').text(addr.postalCode + ' ' + addr.city))
            );
            if (this.localizer_.localizedProviderValue(this.branch_.provider, 'websiteUrl') && (this.configuration_ == 'list')) {
                $('<a></a>').
                prop('href',this.localizer_.localizedProviderValue(this.branch_.provider, 'websiteUrl')).
                attr('target', '_blank').
                text(this.localizer_.localizedProviderValue(this.branch_.provider, 'websiteUrl')).
                appendTo(address);
            }

            if (this.configuration_ == 'detail') {
                address.append($('<div /><br />'));
                if (this.branch_.contactEmail) {
                    $('<a></a>').
                    prop('href', 'mailto:' + this.branch_.contactEmail).
                    text(this.branch_.contactEmail).
                    appendTo(address);
                }
                if (this.branch_.contactPhone) {
                    $('<span />').
                    text(this.branch_.contactPhone).
                    appendTo(address);
                }
            }

            if (this.configuration_ == 'list') {
                var viewDiscountBtn = $('<button></button>').
                addClass('dm-btn').
                addClass('dm-btn-active').
                addClass('infowindow-button').
                data('dm-provider-id', this.branch_.provider.id).
                text(discount_search.list_images_viewDiscount).
                appendTo(
                    $('<a />').
                    attr('href', window.dmGetProviderUrl(this.branch_.provider, this.cardType)).
                    appendTo(content)
                );
            }
            return content;
        }
    });

    /**
     * Class responsible for rendering map of the providers.
     */
    window.DmMapRenderer = function(element, localizer, categories, type) {
        this.constructor.call(this, element, localizer, categories, type);
    }
    $.extend(DmMapRenderer.prototype, {

        localizer: null,
        categories: null,

       /**
         * Construct the renderer object.
         */
        constructor: function(element, localizer, categories, type) {
            this.element = $(element);
            this.localizer = localizer;
            this.categories = categories;

            this.map = null;
            this.mapElement = null;
            this.infowindowConfiguration = type;
            this.markers = {};

            this.mapElement = $('<div></div>').
                addClass('dm-provider-map');
            this.element.append(this.mapElement);
            if (this.infowindowConfiguration == 'list') {
                this.createLegend();
            }
            if (navigator.geolocation) {
                navigator.geolocation.getCurrentPosition(
                    this.showUserPosition.bind(this),
                    this.handlePositionErrors.bind(this)
                );
            } else {
                console.log('No geolocation support, should implement ajax-based one.');
            }

            var mapOptions = {
                center: new google.maps.LatLng(discount_search.initial_lat, discount_search.initial_lon),
                zoom: +discount_search.initial_zoom
            };
            this.map = new google.maps.Map(
                this.mapElement.get(0),
                mapOptions
            );
            var that = this;
            google.maps.event.addListener(this.map, 'idle', function() {
                that.onChange()
            });
        },

        createLegend: function() {
            var legend = $('<div></div>') .
                    addClass('dm-map-legend').
                    appendTo(this.element);
            $('<h2></h2>') .
                    text(discount_search.list_map_legend) .
                    appendTo(legend);
            var categoriesWrapper = $('<div></div>') .
                addClass('dm-map-legend-categories') .
                addClass('clearfix') .
                appendTo(legend);

            var categories = ['travel', 'transportation', 'eshop', 'festival', 'foodAndDrink', 'shopping', 'culture', 'skiResort', 'sport', 'accommodation', 'education', 'funAndRelax', 'other'];
            for (var i = 0; i < categories.length; i++) {
                var categoryWrapper = $('<div></div>').addClass('dm-map-legend-categories-category').appendTo(categoriesWrapper);
                var imgWrapper = $('<dd></dd>').appendTo(categoryWrapper);
                var img = $('<span></span>') .
                        addClass('dm-category-icon') .
                        addClass('dm-category-icon-' + categories[i]).
                        appendTo(imgWrapper);
                var text = $('<dt></dt>').
                        text(discount_search['category_' + categories[i]]).
                        appendTo(categoryWrapper);
            };
        },

        onChange: function() {
        },

        showUserPosition: function(position) {
            if (this.infowindowConfiguration == 'list') {
                this.map.setCenter(
                    new google.maps.LatLng(
                        position.coords.latitude,
                        position.coords.longitude
                    )
                );
                var accuracy = position.coords.accuracy;
                var zoom = 11
                if (accuracy < 16000 && accuracy > 13000)
                    zoom = 12;
                else if (accuracy < 13000 && accuracy > 9750)
                    zoom = 13;
                else if (accuracy < 9750 && accuracy > 6500)
                    zoom = 14;
                else if (accuracy < 6500 && accuracy > 3250)
                    zoom = 15;
                else if (accuracy < 3250 && accuracy > 150)
                    zoom = 16;
                else if (accuracy < 150)
                    zoom = 17;

                this.map.setZoom(zoom);
            }
        },

        handlePositionErrors: function(error) {
            switch(error.code) {
                case error.PERMISSION_DENIED:
                    // user denied permission to geolocation, so do nothing
                    break;
                case error.POSITION_UNAVAILABLE:
                    // TODO: ask server for location estimate
                    break;
                case error.TIMEOUT:
                    // should not happen
                    break;
                case error.UNKNOWN_ERROR:
                    // unknown error, log this somehow?
                    break;
            }
        },

        getZoom: function() {
            return this.map.getZoom();
        },

        isBoundingBox: function() {
            var bounds = this.map.getBounds();
            if (bounds == null || typeof bounds == 'undefined') {
                return false;
            } else {
                return true;
            }
        },

        getBoundingBox: function() {
            var bounds = this.map.getBounds();
            return bounds.getNorthEast().lat() + "," + bounds.getSouthWest().lng() +
                    ":" +
                    bounds.getSouthWest().lat() + "," + bounds.getNorthEast().lng();
        },

        takeControl: function() {
            this.element.show();
            google.maps.event.trigger(this.map, 'resize')
        },

        /*
         * Updates map with new providers nad stuff.
         */
        replace: function(items, categoryId, cardType, searchText) {
            if (this.categoryId != categoryId || this.cardType != cardType || this.searchText != searchText) {
                this.deleteAllMarkers.bind(this)();
            }
            this.showMarkers.bind(this)(items);
            this.categoryId = categoryId;
            this.cardType = cardType;
            this.searchText = searchText;
        },

        categoryTypeToX: function (categoryType) {
            var x = null;
            switch (categoryType) {
                case 'other':
                    x = 384;
                    break;
                case 'funAndRelax':
                    x = 352;
                    break;
                case 'education':
                    x = 320;
                    break;
                case 'accommodation':
                    x = 288;
                    break;
                case 'sport':
                    x = 256;
                    break;
                case 'skiResort':
                    x = 224;
                    break;
                case 'culture':
                    x = 192;
                    break;
                case 'shopping':
                    x = 160;
                    break;
                case 'foodAndDrink':
                    x = 128;
                    break;
                case 'festival':
                    x = 96;
                    break;
                case 'eshop':
                    x = 64;
                    break;
                case 'transportation':
                    x = 32;
                    break;
                case 'travel':
                    x = 0;
                    break;
            }
            return x;
        },

        categoryTypeToIcon: function(categoryType) {
            var y = 0;
            var x = this.categoryTypeToX(categoryType);
            return {
                origin: new google.maps.Point(x, y),
                size: new google.maps.Size(32, 37),
                url: discount_search.categories_sprites_url
            };
        },


        getCategoryTypeForDiscounts: function(discounts) {
            var categories = [];
            for (var i = 0; i < discounts.length; i++) {
                for (var j = 0; j < discounts[i].categories.length; j++) {
                    categories.push(discounts[i].categories[j].id);
                };
            };
            var pickedCategoryId = categories.sort(function(a, b) {return a-b})[0];
            if (this.categories[pickedCategoryId]) {
                return this.categories[pickedCategoryId]['icon'];
            } else {
                return 'other';
            }
        },

        showMarkers: function(items) {
            for (var i = items.length - 1; i >= 0; --i) {
                if (!(items[i].id in this.markers)) {
                    if (!items[i].location || !items[i].discounts || items[i].discounts.length == 0 || items[i].location.active) {
                        continue;
                    }
                    var marker = new google.maps.Marker({
                        position: new google.maps.LatLng(
                            items[i].location.lat,
                            items[i].location.lon
                        ),
                        map: this.map,
                        icon: this.categoryTypeToIcon(this.getCategoryTypeForDiscounts(items[i].discounts))
                    });
                    google.maps.event.addListener(marker, 'click', this.mouseOver.bind(this, items[i], marker));
                    this.markers[items[i].id] = marker;
                }
            };
            if (this.infowindowConfiguration == 'detail') {
                var latlngbounds = new google.maps.LatLngBounds();
                var count = 0;
                for (var id in this.markers) {
                    latlngbounds.extend(this.markers[id].getPosition());
                    ++count;
                }
                var fixZoom = function() {
                    google.maps.event.addListenerOnce(
                            this.map,
                            'bounds_changed',
                            function (e) {
                                if (this.getZoom() > 17) {
                                    this.setZoom(17);
                                }
                            }
                    );
                }.bind(this);
                this.map.fitBounds(latlngbounds);
                fixZoom();
                google.maps.event.addListenerOnce(
                        this.map,
                        'idle',
                        function (e) {
                            this.fitBounds(latlngbounds);
                            fixZoom();
                        }
                );
            }
        },

        mouseOver: function(branch, marker, e) {
            if (this.infowindow_cur) {
                this.infowindow_cur.setMap(null);
            }
            if (! marker.infoWindow ) {
                marker.infoWindow = new InfoWindow(branch, this.infowindowConfiguration,  marker, this.map, this.localizer, this.cardType);
                this.infowindow_cur = marker.infoWindow;
            }
            this.map.panTo(marker.getPosition());
        },

        deleteAllMarkers: function() {
            for (var key in this.markers) {
                this.markers[key].setMap(null);
            };
            this.markers = {};
        }

    });

    window.InfoWindow = function(branch, configuration, marker, map, localizer, cardType) {
        this.branch_ = branch;
        this.configuration_ = configuration;
        this.marker_ = marker;
        this.map_ = map;
        this.localizer_ = localizer;
        this.cardType = cardType;

        this.div_ = null;

        this.setMap(map);
    };

    if (discount_search.map_provider == 'google') {
        window.InfoWindow.prototype = new google.maps.OverlayView();
    }
    window.InfoWindow.prototype.is_detail_ = function() {
        return this.configuration_ == 'detail';
    };
    window.InfoWindow.prototype.is_list_ = function() {
        return this.configuration_ == 'list';
    };
    window.InfoWindow.prototype.onAdd = function() {
        var div = $('<div></div>').
                addClass('infowindow');

        var wrapper = $('<div>').
                addClass('infowindow-wrapper').
                appendTo(div);

        google.maps.event.addDomListener(
                wrapper.get(0),
                'click',
                function (e) {
                    e.cancelBubble = true;
                    if (e.stopPropagation) {
                        e.stopPropagation();
                    }
                }
        );

        var closeBtn = $('<span></span>').
                addClass('close-btn').
                text('x').
                appendTo(wrapper);
        google.maps.event.addDomListener(
                closeBtn.get(0),
                'click',
                function() {
                    this.setMap(null);
                }.bind(this)
        );

        var content = $('<div>').
                addClass('infowindow-content').
                appendTo(wrapper);

        var name = $('<h1>').
                text(this.localizer_.localizedBranchValue(this.branch_, 'name')).
                appendTo(content);

        var discounts = $('<span></span>').
                addClass('infowindow-discounts').
                appendTo(content);

        var wrapper;
        if (this.is_detail_()) {
            wrapper = $('<ul />').appendTo(discounts);
        } else if (this.is_list_()) {
            wrapper = discounts;
        }
        for (var i = 0; i < this.branch_.discounts.length; i++) {
            var currentDate = new Date().getTime();
            var validSince = new Date(this.branch_.discounts[i].validSince).getTime();
            var validUntil = new Date(this.branch_.discounts[i].validUntil).getTime();

            if (!this.branch_.discounts[i].active || (validSince > currentDate || validUntil < currentDate)) {
                continue;
            }
            var discount;
            if (this.is_detail_()) {
                discount = $('<li />');
            } else if (this.is_list_()) {
                discount = $('<span />').
                        addClass('infowindow-discount');
            }
            discount.
                    text(this.localizer_.localizedDiscountValue(this.branch_.discounts[i], 'name')).
                    appendTo(wrapper);
        };

        var address = $('<div></div>').
                addClass('infowindow-address').
                html('<strong>' + discount_search.infowindow_address + '</strong><br />').
                appendTo(content);
        var addr = this.branch_.address;
        address.append(
            $('<span></span>').
                    append($('<span></span>').text(addr.address1 + (addr.address2 ? ', ' + addr.address2 : ''))).
                    append($('<br />')).
                    append($('<span></span>').text(addr.postalCode + ' ' + addr.city))
        );
        if (this.localizer_.localizedProviderValue(this.branch_.provider, 'websiteUrl') && this.is_list_()) {
            $('<a></a>').
                prop('href',this.localizer_.localizedProviderValue(this.branch_.provider, 'websiteUrl')).
                attr('target', '_blank').
                text(this.localizer_.localizedProviderValue(this.branch_.provider, 'websiteUrl')).
                appendTo(address);
        }
        if (this.is_detail_()) {
            address.append($('<div /><br />'));
            if (this.branch_.contactEmail) {
                $('<a></a>').
                    prop('href', 'mailto:' + this.branch_.contactEmail).
                    text(this.branch_.contactEmail).
                    appendTo(address);
            }
            if (this.branch_.contactPhone) {
                $('<span />').
                    text(this.branch_.contactPhone).
                    appendTo(address);
            }
        }

        if (this.is_list_()) {
            var viewDiscountBtn = $('<button></button>').
                    addClass('dm-btn').
                    addClass('dm-btn-active').
                    addClass('infowindow-button').
                    data('dm-provider-id', this.branch_.provider.id).
                    text(discount_search.list_images_viewDiscount).
                    appendTo(
                            $('<a />').
                                    attr('href', window.dmGetProviderUrl(this.branch_.provider, this.cardType)).
                                    appendTo(content)
                    );

            google.maps.event.addDomListener(
                    viewDiscountBtn.get(0),
                    'click',
                    function (e) {
                        viewDiscountBtn.data('dm-provider-id', this.branch_.provider.id);
                        viewDiscountBtn.trigger('click');
                        e.cancelBubble = true;
                        if (e.stopPropagation) {
                            e.stopPropagation()
                        };
                        return false;
                    }.bind(this)
            );
        }

        this.div_ = div;

        this.getPanes().floatPane.appendChild(this.div_.get(0));
    };
    window.InfoWindow.prototype.draw = function() {
        var projection = this.getProjection();
        var anchor = projection.fromLatLngToDivPixel(this.marker_.getPosition());

        var div = this.div_;

        var oldDisplay = div.css('display');
        var oldPosition = div.css('position');
        div.css('display', 'hidden');
        div.css('position', 'absolute');

        var height = div.height();
        if (!this.arrow_) {
            var arrow = $('<div></div>').
                    addClass('infowindow-arrow').
                    offset({top: height / 2 - 20.5, left: div.width() - 5}).
                    appendTo(this.div_);
            this.arrow_ = arrow;
            google.maps.event.addDomListener(
                    arrow.get(0),
                    'click',
                    function (e) {
                        e.cancelBubble = true;
                        if (e.stopPropagation) {
                            e.stopPropagation();
                        }
                    }
            );
        }

        div.css('display', oldDisplay);
        div.css('position', oldPosition);

        div.
                css('left', anchor.x - 245 + 'px').
                css('top', anchor.y - (height / 2 + 18.5) + 'px');
    };
    window.InfoWindow.prototype.onRemove = function() {
        var div = this.div_.get(0);
        div.parentNode.removeChild(div);
        this.div_ = null;
        this.marker_.infoWindow = null;
    };

    /**
     * Class responsible for rendering detail of the providers.
     */
    window.DmLocalizer = function(locale) {
        this.constructor.call(this, locale);
    }
    $.extend(DmLocalizer.prototype, {

        /**
         * Target locale.
         */
        locale: null,

        /**
         * Construct the localizer object.
         * @pararm locale Locale to be used for translation.
         */
        constructor: function(locale) {
            this.locale = locale;

            this.decimal_separator = discount_search.number_format[discount_search.number_format.length - 2];
            this.thousand_separator= discount_search.number_format[discount_search.number_format.length - 6];
            if (!isNaN(this.thousand_separator) && !isNaN(parseInt(this.thousand_separator))) {
                this.thousand_separator = '';
            }
        },

        returnAsMoney: function(amount) {
            if (discount_search.currency_pos == 1) {
                return discount_search.currency + ' ' + this.formatNumber(amount);
            } else if (discount_search.currency_pos == 2) {
                return this.formatNumber(amount) + ' ' + discount_search.currency;
            } else {
                throw "Should not get here.";
            }
        },

        formatNumber: function(n, decimal) {
            window.formatNumber = this.formatNumber.bind(this);
            decimal = typeof decimal !== 'undefined' ? decimal : 2;
            var d = Math.abs(n - Math.floor(n)).toFixed(decimal);
            var i = Math.floor(n);
            var i_ = i + '';
            var result = '';
            var is = [];
            for (var i = i_.length - 3; i >= 0; i -= 3) {
                is.unshift(i_.substr(i, 3));
            };
            is.unshift(i_.substr(0, i_.length % 3));
            for (var i = 0; i < is.length; i++) {
                result += is[i];
                if (is.length -1 != i) {
                    result += this.thousand_separator;
                }
            };
            if (d != 0) {
                result += this.decimal_separator + (d + '').substr(2);
            }
            return result;
        },

        /**
         * Get translation message. Returns key if message does not exist.
         * @param string key Key of the translation message.
         */
        message: function(key) {
            if (discount_search[key]) {
                return discount_search[key];
            }
            return key;
        },

        /**
         * Strip HTML tags from given content and return plain text.
         *
         * @param {string} content
         * @returns {string}
         */
        stripTags: function(content) {
            var div = document.createElement('div');
            div.innerHTML = content;

            return div.innerText;
        },

        /**
         * Get provider localized value.
         * @param provider Provider entity.
         * @param value Name of the value that should be rendered.
         */
        localizedProviderValue: function(provider, value) {
            var l10n = this.getL10n(provider['providerL10ns']);
            var content = (l10n != null && l10n[value]) ? l10n[value] : provider[value];

            // If is description, return stripped text.
            if (value === 'description') {
                content = this.stripTags(content);
            }

            return content;
        },

        /**
         * Get provider discount value.
         * @param provider Discount entity.
         * @param value Name of the value that should be rendered.
         */
        localizedDiscountValue: function(discount, value) {
            var l10n = this.getL10n(discount['discountL10ns']);
            if (l10n != null && l10n[value]) {
                return this.stripTags(l10n[value]);
            }
            // This is special case
            if (value == 'otherRestriction') {
                return discount['restrictions']['other'];
            }
            return this.stripTags(discount[value] || '');
        },

        localizedDiscountType: function(discount) {
            var type = discount['type'];
            if (type == 'ABSOLUTE') {
                return this.returnAsMoney(discount['value']) + ' ' + this.message('discount_discount');
            } else if (type == 'PERCENTAGE') {
                return discount['value'] + '% ' + this.message('discount_discount');
            } else {
                return this.message('discount_type_' + type);
            }
        },

        localizedBranchValue: function(branch, value) {
            var l10n = this.getL10n(branch['branchL10ns']);
            if (l10n != null && l10n[value]) {
                return l10n[value];
            }

            return branch[value];
        },

        /**
         * Get category localized value.
         * @param category Category entity.
         * @param value Name of the value that should be rendered.
         */
        localizedCategoryValue: function(category, value) {
            var l10n = this.getL10n(category['categoryL10ns']);
            if (l10n != null && l10n[value]) {
                return l10n[value];
            }

            return category[value];
        },

        /**
         * Get localization object for actual locale. Assumes that l10n object has field 'languageCode'.
         * @param array l10ns List of localization objects to be searched.
         */
        getL10n: function(l10ns) {
            for (var i = 0; i < l10ns.length; i++) {
                if (l10ns[i]['languageCode'] == this.locale) {
                    return l10ns[i];
                }
            }
            return null;
        },

        /**
         * Escape values so is can be used safely in html.
         * XXX: This is not the best place where to define it but most
         * suitable in the context of application for now.
         * @param string value Value to be escaped.
         * @return string Escaped value.
         */
        escape: function (value) {
            // Hacked through jQuery.
            return $('<span>').text(value).html();
        }

    });

    // ~ Detail renderer.

    /**
     * Class responsible for rendering detail of the providers.
     */
    window.DmDetailRenderer = function(wrapper, localizer, dmClient, categories, cardTypes) {
        this.constructor.call(this, wrapper, localizer, dmClient, categories, cardTypes);
    }
    $.extend(DmDetailRenderer.prototype, {

        wrapper: null,
        result: null,
        providerId: null,
        detail: null,
        localizer: null,
        dmClient: null,
        daysMap: {
            MONDAY: discount_search.detail_restrictions_monday,
            TUESDAY: discount_search.detail_restrictions_tuesday,
            WEDNESDAY: discount_search.detail_restrictions_wednesday,
            THURSDAY: discount_search.detail_restrictions_thursday,
            FRIDAY: discount_search.detail_restrictions_friday,
            SATURDAY: discount_search.detail_restrictions_saturday,
            SUNDAY: discount_search.detail_restrictions_sunday
        },
        categories: null,
        cardTypes: null,

        /**
         * Construct the renderer object.
         * @pararm wrapper Wrapper of the element where results should be rendered.
         */
        constructor: function(wrapper, localizer, dmClient, categories, cardTypes) {
            this.wrapper = wrapper;
            this.localizer = localizer;
            this.dmClient = dmClient;
            this.categories = categories;
            this.cardTypes = cardTypes;
        },

        discountActive: function (discount) {
            var validSince = new Date(discount.validSince.split('+')[0]).getTime();
            var validUntil = discount.validUntil ? new Date(discount.validUntil.split('+')[0] + 'Z').getTime() : null;
            return discount.active && validSince <= new Date().getTime() &&
                    (!discount.validUntil || validUntil >= new Date().getTime());
        },

        /**
         * Render the provider.
         * @param provider Provider details.
         */
        render: function(provider, cardType) {
            var seoTitle = this.localizer.localizedProviderValue(provider, 'seoTitle');
            if (seoTitle && seoTitle !== 'undefined') {
                document.title = discount_search.blog_name + " | " + seoTitle;
            } else {
                document.title = discount_search.blog_name + " | " + this.localizer.localizedProviderValue(provider, 'name');
            }

            var metaDescription = this.localizer.localizedProviderValue(provider, 'metaDescription');
            if (metaDescription && metaDescription !== 'undefined') {
                $("head").append("<meta name='description' content='" + metaDescription + "' />");
            } else {
                metaDescription = this.localizer.localizedProviderValue(provider, 'shortDescription');
                $("head").append("<meta name='description' content='" + metaDescription + "' />");
            }

            var cardTypes = this.cardTypes;
            if (typeof cardType !== 'undefined' && cardType !== null) {
                cardTypes = [cardType];
            }
            if (this.providerId == provider.id) {
                return;
            }
            var that = this;
            this.result = $("<div id='dm-provider-detail'></div>");
            var leftColumn = $("<div />").addClass("dm-box").appendTo(
                    $("<div />").addClass("dm-left-column").appendTo(this.result)
            );
            leftColumn.append(
                     $("<h1 />").addClass('dm-header').text(this.localizer.localizedProviderValue(provider, 'name'))
            );
            var description = $("<p />").
                    text(this.localizer.localizedProviderValue(provider, 'description')).
                    addClass("dm-description").
                    appendTo(leftColumn);

            $("<h2 />").addClass('dm-header').
                    text(this.localizer.message('detail_offeredDiscounts')).
                    appendTo(leftColumn);

            if (provider.active) {
                var discounts = $("<div />").
                        addClass("dm-discount-list").
                        appendTo(leftColumn);
                provider.discounts.forEach(function (discount) {
                    if (that.discountActive(discount) && that.isAllowedCardTypes(discount.cardTypes, that.cardTypes)) {
                        discounts.append(that.renderDiscount(discount));
                    }
                });

                if (provider.branches.length > 0) {
                    var activeCount = 0;
                    for (var i = 0; i < provider.branches.length; i++) {
                        if (provider.branches[i].active && provider.branches[i].visible) {
                            for (var j = 0; j < provider.discounts.length; j++) {
                                if (provider.discounts[j].branches.length > 0) {
                                    if (this.discountActive(provider.discounts[j])) {
                                        ++activeCount;
                                    }
                                }
                            }
                        }
                    }
                    if (activeCount > 0) {
                        if (activeCount == 1) {
                            leftColumn.append($('<h2 />').addClass('dm-header').
                                    text(discount_search.detail_map_header)
                            );
                        } else {
                            leftColumn.append($('<h2 />').addClass('dm-header').
                                    text(discount_search.detail_map_header_s)
                            );
                        }

                        var mapElem = $('<div />').addClass('dm-results');
                        mapElem.appendTo(leftColumn);
                        if (discount_search.map_provider == 'google') {
                            var mapRenderer = new DmMapRenderer(mapElem, this.localizer, this.categories, 'detail');
                        } else {
                            var mapRenderer = new DmMapRendererStreet(mapElem, this.localizer, this.categories, 'detail');
                        }
                        var activeBranches = [];
                        for (var i = 0; i < provider.branches.length; i++) {
                            if (provider.branches[i].active && provider.branches[i].visible) {
                                var branch = provider.branches[i];
                                branch.provider = provider;
                                branch.discounts = [];
                                for (var j = 0; j < provider.discounts.length; j++) {
                                    for (var k = 0; k < provider.discounts[j].branches.length; k++) {
                                        if (branch.id == provider.discounts[j].branches[k].id && this.discountActive(provider.discounts[j])) {
                                            branch.discounts.push(provider.discounts[j]);
                                        };
                                    };
                                };
                                if (branch.discounts.length > 0) {
                                    activeBranches.push(branch);
                                }
                            }
                        };
                        mapRenderer.replace.bind(mapRenderer)(activeBranches);
                    }
                }
            } else {
                var discountsNotAvailable = $("<p />").
                        text(this.localizer.message('discounts_are_not_available_right_now')).
                        addClass("dm-discounts-not-available").
                        appendTo(leftColumn);
            }

            var commentElem = $('<div />').
                    addClass('dm-comments-wrapper').
                    appendTo(leftColumn);
            var comments = new DmCommentModule(commentElem, provider, that.localizer, that.dmClient);

            var rightColumn = $("<div />").addClass("dm-right-column");

            var providerInfo = $("<div />").
                    addClass("dm-box").
                    addClass("dm-box-bordered").
                    addClass('dm-provider-info');

            var logoUrl = null;
            $.each(provider.logo, function(ix, image) {
                if (image.type == "SMALL") {
                    logoUrl = image.url;
                    return false;
                }
            });
            if (!logoUrl && provider.logoUrl) {
                logoUrl = provider.logoUrl;
            }
            if (!logoUrl) {
                logoUrl = discount_search['plugins_url'] + '/resources/alive-dm/img/isic-image.png';
            }
            var providerLogo = $('<img />').
                    addClass("dm-provider-detail-logo").
                    attr("src", logoUrl);

            var rating = new DmRatingModule(this.providerId, provider.providerStats.averageRating, 'dm-provider-current-rating', function() {
                that.dmClient.rateProvider(
                    that.providerId, this.getValue(),
                    function (d) {
                        this.ratingId = d.id;
                    }.bind(this)
                );
            });
            var providerRating = rating.render();

            var providerLink = $("<a></a>").
                    attr('href', this.localizer.localizedProviderValue(provider, 'websiteUrl')).
                    attr('target', '_blank');

            providerLink.append(providerLogo);
            providerInfo.append(providerLogo);
            providerInfo.append(providerRating);

            if (this.localizer.localizedProviderValue(provider, 'websiteUrl')) {
                var strippedUrl;
                if (provider.websiteUrl.indexOf("https://") == 0) {
                    strippedUrl =this.localizer.localizedProviderValue(provider, 'websiteUrl').substring(8);
                }
                if (provider.websiteUrl.indexOf("http://") == 0) {
                    strippedUrl = this.localizer.localizedProviderValue(provider, 'websiteUrl').substring(7);
                }
                var website = $("<span />").
                        text(this.localizer.message('detail_website') + ': ').
                        append($("<a />").
                                attr('href', this.localizer.localizedProviderValue(provider, 'websiteUrl')).
                                attr('target', '_blank').
                                text(strippedUrl).
                                attr('rel', 'sponsored')
                        ).addClass("dm-contact-line");
                providerInfo.append(website);
                providerInfo.append("<hr />");
            }

            if (provider.contactEmail) {
                var email = $("<span />").
                        text(this.localizer.message('detail_email') + ': ').
                        append($('<a />').
                                attr('href', 'mailto:' + provider.contactEmail).
                                text(provider.contactEmail)
                        ).addClass("dm-contact-line");
                providerInfo.append(email);
                providerInfo.append("<hr />");
            }

            if (provider.contactPhone) {
                var phone = $('<span />').
                        text(this.localizer.message('detail_phone') + ': ').
                        append($('<span />').
                                text(provider.contactPhone)
                        ).addClass("dm-contact-line");
                providerInfo.append(phone);
            }

            var twitterText = encodeURIComponent(
                    discount_search.isic_discount + ' ' +
                    this.localizer.localizedProviderValue(provider, 'name') + ' ' +
                    dmGetProviderUrl(provider) + ' ' +
                    '#isic' + ' via @' + discount_search.twitter_account
            );
            var socialNetworks = $("<div />").addClass("dm-social-links");

            var facebookLink = $('<a href="https://www.facebook.com/sharer/sharer.php?u=' + encodeURIComponent(dmGetProviderUrl(provider)) + '" class="dm-social-network"></a>');
            var facebook = $("<div>Share</div>").addClass("dm-btn-social").addClass("dm-btn-facebook");
            facebookLink.append(facebook);
            socialNetworks.append(facebookLink);

            var twitterLink = $('<a href="https://twitter.com/intent/tweet/?url=&text=' + twitterText + '" class="twitter-share-button dm-social-network"></a>');
            var twitter = $("<div>Tweet</div>").addClass("dm-btn-social").addClass("dm-btn-twitter");
            twitterLink.append(twitter);
            socialNetworks.append(twitterLink);

            providerInfo.append(socialNetworks);
            rightColumn.append(providerInfo);

            var providerPhotos = $("<div></div>").
                    addClass("dm-box").
                    addClass("dm-box-bordered").
                    append("<h2>" + this.localizer.message('detail_photos_videos') + "</h2>");

            var photos = $("<div></div>").addClass("dm-photos");

            var addWhitePlus = function(photoWrapper) {
                photoWrapper.append(
                    $('<img />').
                            addClass('dm-photo-white-plus-overlay').
                            attr('src', discount_search.white_plus_overlay_url)
                );
            };
            var i = 0;
            provider.videos.forEach(function(video) {
                i++;
                photos.append(that.renderPhoto(video, i > 8));
            });
            if (provider.image) {
                var photo = {
                    photoUrl: provider.imageUrl,
                    image: provider.image,
                    name: this.localizer.localizedProviderValue(provider, 'name')
                };
                photos.append(this.renderPhoto(photo, false));
                i++;
            }
            provider.photos.forEach(function(photo) {
                i++;
                photos.append(that.renderPhoto(photo, i > 8));
            });
            providerPhotos.append(photos);
            rightColumn.append(providerPhotos);
            var keywords = this.localizer.localizedProviderValue(provider, 'keywords');
            var tag;
            if (keywords && keywords != "") {
                var providerTags = $("<div></div>").addClass("dm-box").addClass("dm-box-bordered");
                    providerTags.append("<h2>" + this.localizer.message('detail_tags') + "</h2>");
                    keywords.forEach(function(keyword) {
                        tag = $('<a />').
                            attr('href', window.dmGetDiscountSearchUrl({'dmText': keyword})).
                            addClass('dm-tag').
                            data('dm-tag', keyword).
                            text(keyword);
                        if (keyword.length >= 20) {
                            tag.attr('title', keyword);
                        }
                        providerTags.append(tag);
                    });
                rightColumn.append(providerTags);
            }

            var categories = [];
            provider.discounts.forEach(function(discount) {
               discount.categories.forEach(function(category) {
                    categories.push(category.id);
                });
            });
            this.dmClient.getProviders({
                categoryIds: categories,
                cardTypes: cardTypes,
                limit: 10
            }, function(result){
                if (result.count > 1) {
                    var similarDiscounts = $("<div></div>").addClass('dm-box').addClass('dm-box-bordered').addClass('dm-similar-discounts');
                        similarDiscounts.append("<h2>" + that.localizer.message('detail_similarDiscounts') + "</h2>");
                        result.items.forEach(function(similarProvider) {
                            if (similarProvider.id == provider.id) {
                                return;
                            }
                            var providerLink = $('<a />').attr('href', window.dmGetProviderUrl(similarProvider)).
                                    appendTo(similarDiscounts)
                            var providerBox = $("<div>") .
                                    addClass('dm-similar-provider').addClass('clearfix').
                                    attr('data-dm-provider-id', similarProvider.id);
                            providerLink.append(providerBox)
                            var logoUrl = discount_search['plugins_url'] + '/resources/alive-dm/img/isic-image.png';
                            $.each(similarProvider.logo, function(ix, image) {
                                if (image.type == "SMALL") {
                                    logoUrl = image.url;
                                    return false;
                                }
                            });
                            if (!logoUrl && similarProvider.logoUrl) {
                                logoUrl = similarProvider.logoUrl;
                            }
                            var providerLogo = $("<div>").addClass("dm-similar-provider-logo-wrapper").append($("<img />").attr('src', logoUrl));
                            providerBox.append(providerLogo);
                            var providerInfoWrapper = $("<div>").addClass("dm-similar-provider-info-wrapper");
                            providerBox.append(providerInfoWrapper);
                            providerInfoWrapper.append(
                                    $('<h3 />').text(that.localizer.localizedProviderValue(similarProvider, 'name'))
                            );
                            providerInfoWrapper.append(
                                    $('<p />').
                                            text(that.localizer.localizedDiscountType(similarProvider.discounts[0]))
                            );
                            similarDiscounts.append(providerLink);
                        });
                    rightColumn.append(similarDiscounts);
                }
            });
            this.result.append(rightColumn);
            this.result.append($("<div />").addClass("clearer"));
            this.wrapper.children().hide();
            this.wrapper.append(this.result);
            if (mapRenderer) { //If theres are no branches, map renderer is not initialized.
                mapRenderer.takeControl();
            }
            this.providerId = provider.id;
        },

        /**
         * Check whether the card type is allowed for this plugin.
         * @param cardTypes array|string Card type or list of card types that should be checked.
         * @param allowedCardTypes Card types to bechecked. Can be null. If null, defaults will be used.
         * @return True if at least one of the card types is among the allowed card types. False otherwise.
         */
        isAllowedCardTypes: function(cardTypes, allowedCardTypes) {
            var allowed = false;
            if (allowedCardTypes == null) {
                allowedCardTypes = this.cardTypes;
            }
            if (!Array.isArray(cardTypes)) {
                cardTypes = [cardTypes];
            }
            cardTypes.forEach(function(cardType) {
                allowedCardTypes.forEach(function(allowedCardType) {
                    if (cardType.toLowerCase() == allowedCardType.toLowerCase()) {
                        allowed = allowed || true;
                    }
                });
            });
            return allowed;
        },

        /**
         * Render photo of the provider.
         */
        renderPhoto: function(photo, hidden) {
            var addWhitePlus = function(photoWrapper) {
                photoWrapper.append(
                    $('<img />').
                            addClass('dm-photo-white-plus-overlay').
                            attr('src', discount_search.white_plus_overlay_url)
                );
            };
            var addVideoPlay = function(photoWrapper) {
                photoWrapper.append(
                    $('<img />').
                            addClass('dm-video-play-overlay').
                            attr('src', discount_search.white_play_overlay_url)
                );
            };
            var title = this.localizer.escape(photo.name) + (photo.description ? ' - ' + this.localizer.escape(photo.description) : '');
            var photoUrl = null;
            if (photo.image !== undefined) {
                $.each(photo.image, function(ix, image) {
                    if (image.type == "LARGE" || image.type == "NORMAL") {
                        photoUrl = image.url;
                        return null;
                    }
                });
            }
            if (photoUrl == null) {
                if (photo.type !== undefined && photo.type.indexOf('VIDEO') === 0) {
                    photoUrl = photo.videoUrl;
                } else {
                    photoUrl = photo.photoUrl;
                }
            }
            var photoLink = $("<a></a>").
                    attr('rel', 'dm-lightbox[a]').
                    attr('href', photoUrl).
                    attr('title', title);
            if (hidden) {
                photoLink.addClass('hidden');
            }
            var photoWrapper = $("<div></div>").addClass("dm-photo");
            addWhitePlus(photoWrapper);
            var thumbnailUrl = null;
            if (photo.image !== undefined) {
                $.each(photo.image, function(ix, image) {
                    if (image.type == "THUMB") {
                        thumbnailUrl = image.url;
                        return false;
                    }
                });
            }
            if (thumbnailUrl === null) {
                if (photo.type !== undefined && photo.type.indexOf('VIDEO') === 0) {
                    thumbnailUrl = photo.imageUrl;
                } else {
                    thumbnailUrl = photo.photoUrl;
                }
            }
            var photoImage = $("<img />").
                attr('src', thumbnailUrl).
                attr('alt', photo.name).
                addClass('dm-photo-img').
                load(function() {
                    if (this.width >= this.height) {
                        $(this).addClass("dm-orientation-landscape");
                    } else {
                        $(this).addClass("dm-orientation-portrait");
                    }
                    $(photoWrapper).dmBlurImage();
                });
            photoWrapper.append(photoImage);
            if (photo.type !== undefined && photo.type.indexOf('VIDEO') === 0) {
                addVideoPlay(photoWrapper);
            }
            photoLink.append(photoWrapper);
            return photoLink;
        },

        isValidUrl: function(string) {
            try {
                new URL(string);
                return true;
            } catch (_) {
                return false;
            }
        },

        /**
         * Render provider discount.
         *
         * @param discount Discount object.
         * @param Rendered discount.
         */
        renderDiscount: function(discount) {
            var that = this;
            var discountItem = $("<div></div>").addClass("dm-discount-list-item");
            var discountName = $("<h3 />").
                    text(this.localizer.localizedDiscountValue(discount, 'name')).
                    addClass("dm-discount-name");
            discountItem.append(discountName);
            var cardTypes = $("<span></span>").addClass("dm-card-types");
            discount.cardTypes.forEach(function(cardType) {
                if (that.isAllowedCardTypes(cardType)) {
                    var cardTypeLink = (discount_search[cardType.toLowerCase() + '_url']) ? $("<a></a>").attr('href', (that.isValidUrl(discount_search[cardType.toLowerCase() + '_url'])) ? discount_search[cardType.toLowerCase() + '_url'] : window.location.protocol + '//' + window.location.host + '/' + discount_search[cardType.toLowerCase() + '_url']).attr('target', '_blank') : "";
                    var cardTypeIcon = $("<span></span>").
                            addClass("dm-icon-card-type").
                            addClass("dm-icon-card-type-" + cardType.toLowerCase());

                    if (discount_search[cardType.toLowerCase() + '_url']) {
                        cardTypeLink.append(cardTypeIcon);
                        cardTypes.append(cardTypeLink);
                    } else {
                        cardTypes.append(cardTypeIcon);
                    }

                }
            });
            discountItem.append(cardTypes);
            var discountDescription = $("<p />").text(this.localizer.localizedDiscountValue(discount, 'description'));
            discountItem.append(discountDescription);
            discountItem.append(this.renderRestrictions(discount));
            if (discount.availableOnline) {
                var button = $("<a>" + this.localizer.message('discount_useOnline') + "</a>").
                        attr('target', '_blank').
                        attr('rel', 'sponsored').
                        attr("href", this.localizer.localizedDiscountValue(discount, 'url')).
                        addClass("dm-btn").
                        addClass("dm-btn-use-online").
                        addClass("dm-btn-active");
                discountItem.append(button);

            }
            discountItem.append("<div class='clearer'></div>");
            return discountItem;
        },

        /**
         * Render restictions of the discount.
         *
         * @param discount Discount object.
         * @return Rendered description.
         */
        renderRestrictions: function(discount) {
            var restrictions = discount.restrictions;
            var restrictionList = $("<ul></ul>");
            restrictionList.addClass('dm-provider-detail-discount-restrictions');
            if (restrictions.notForDiscountedItems) {
                restrictionList.append("<li>" + this.localizer.message('discount_restriction_notForDiscountedItems') + "</li>");
            }
            if (restrictions.days.length < 7 || restrictions.timeFrom || restrictions.timeTo) {
                var timeRestriction = this.localizer.message('detail_restrictions_onlyAvailable');
                if (restrictions.days.length < 7) {
                    timeRestriction += " " + this.localizer.message('detail_restrictions_on') +" " + restrictions.days.map(function(day) {
                        return this.daysMap[day];
                    }, this).join(", ");
                }
                if (restrictions.timeFrom || restrictions.timeTo) {
                    var timeFrom;
                    var timeTo;
                    if (restrictions.timeFrom) {
                        timeFrom = {
                            hours: restrictions.timeFrom.substr(0, 2),
                            mins: restrictions.timeFrom.substr(2, 2)
                        }
                    } else {
                        timeFrom = {
                            hours: 0,
                            mins: 0
                        }
                    }
                    if (restrictions.timeTo) {
                        timeTo = {
                            hours: restrictions.timeTo.substr(0, 2),
                            mins: restrictions.timeTo.substr(2, 2)
                        }
                    } else {
                        timeTo = {
                            hours: 23,
                            mins: 59
                        }
                    }
                    timeRestriction += " " + this.localizer.message('detail_restrictions_at') + " " + ('0'+timeFrom.hours).slice(-2) + ":" + ('0'+timeFrom.mins).slice(-2) + " - " + ('0'+timeTo.hours).slice(-2) + ":" + ('0'+timeTo.mins).slice(-2);
                }
                restrictionList.append("<li>" + timeRestriction + "</li>");
            }
            if (restrictions.minAmount) {
                restrictionList.append("<li>" + this.localizer.message('detail_restrictions_minimalCost') + " " + this.localizer.returnAsMoney(restrictions.minAmount) + "</li>");
            }
            if (restrictions.usageInterval) {
                var usageLimit = (restrictions.usageLimit && restrictions.usageLimit > 1)  ?
                        this.localizer.message('detail_restrictions_interval_many').replace('%limit%', restrictions.usageLimit) : this.localizer.message('detail_restrictions_interval_one');
                var intervalMessage;
                if (restrictions.usageInterval == 1) {
                    intervalMessage = this.localizer.message('detail_restrictions_interval_' +  restrictions.usageLimitType.toLowerCase() + '_one');
                } else if (restrictions.usageInterval < 5) {
                    intervalMessage = this.localizer.message('detail_restrictions_interval_' +  restrictions.usageLimitType.toLowerCase() + '_few');
                } else {
                    intervalMessage = this.localizer.message('detail_restrictions_interval_' +  restrictions.usageLimitType.toLowerCase() + '_many');
                }
                intervalMessage = intervalMessage.replace('%limit%', usageLimit).replace('%few%', restrictions.usageInterval).replace('%many%', restrictions.usageInterval);
                restrictionList.append("<li>" + intervalMessage + "</li>");
            }
            if (restrictions.minAge || restrictions.maxAge) {
                var ageRestriction = this.localizer.message('detail_restrictions_onlyAvailable');
                if (restrictions.minAge) {
                    ageRestriction += " " + this.localizer.message('detail_restrictions_from') + " " + restrictions.minAge;
                }
                if (restrictions.maxAge) {
                    ageRestriction += " " + this.localizer.message('detail_restrictions_to') + " " + restrictions.maxAge;
                }
                ageRestriction += " " + this.localizer.message('detail_restrictions_age');
                restrictionList.append("<li>" + ageRestriction + "</li>");
            }
            if (restrictions.other) {
                restrictionList.append(
                        $("<li />").
                                text(this.localizer.localizedDiscountValue(discount, 'otherRestriction'))
                );
            }
            // TODO render each restriction
            return restrictionList;
        },

        /**
         * Render image preview.
         */
        previewImage: function(input, img) {

            if (input.files && input.files[0]) {
                var reader = new FileReader();

                reader.onload = function (e) {
                    img.attr('src', e.target.result);
                }

                reader.readAsDataURL(input.files[0]);
            }
        },

        /**
         * Destroy view of the given provider.
         */
        destroy: function() {
            if (!this.result) {
                return;
            }
            this.result.remove();
            delete this.result;
            this.providerId = null;
            this.wrapper.children().show();
        }

    });


    //~ Searching object.

    /**
     * Class responsible for handling search criteria and their changes.
     */
    window.DmSearching = function(wrapper, ajaxUrl, localizer) {
        this.constructor.call(this, wrapper, ajaxUrl, localizer);
    }
    $.extend(DmSearching.prototype, {

        /**
         * URL for the AJAX calls.
         */
        ajaxUrl: null,

        /**
         * Available card types.
         */
        cardTypes: [],


        //~ Values currently set to the filter.
        text: null,
        categoryId: null,
        cardType: null,


         //~ Search filter fields.
        textInput: null,
        categoryFilter: null,
        cardTypeFilter: null,

        //~ Helper variables.
        searchTimeout: null,

        /**
         * Construct the pagination object.
         * @param wrapper Wrapper of the finder.
         * @param query Array representation of the query parameters.
         */
        constructor: function(wrapper, ajaxUrl, localizer) {
            this.ajaxUrl = ajaxUrl;
            this.init(wrapper, localizer);
        },

        /**
         * Initialize context.
         * @pararm wrapper Wrapper of the finder plugin.
         */
        init: function(wrapper, localizer) {
            var that = this;
            // Initizize category handling.
            this.categoryFilter = $('[data-dm-target="filter-categories"]', this.wrapper);
            this.categoryFilter.on('click', function(event) {
                if (!$(event.target).data("dm-category-id")) {
                    return false; // User didnt click directly on the category
                }
                if (that.categoryId != $(event.target).data("dm-category-id")) {
                    that.setSelectedCategory($(event.target).data("dm-category-id"));
                } else if (that.categoryId == $(event.target).data("dm-category-id")) {
                    that.setSelectedCategory(null);
                }
                that.onChange();
                return false;
            });
            // Initialize card type handling.
            this.cardTypeFilter = $('[data-dm-target="filter-card-types"]', this.wrapper);
            this.cardTypeFilter.on('click', function(event) {
                if (!$(event.target).data("dm-card-type")) {
                    return false; // User didnt click directly on the card type
                }
                if (that.cardType != $(event.target).data("dm-card-type")) {
                    that.setSelectedCardType($(event.target).data("dm-card-type"));
                } else if (that.cardType == $(event.target).data("dm-card-type")) {
                    that.setSelectedCardType(null);
                }
                that.onChange();
                return false;
            });
            // Initialize text input handling.
            this.textInput = $('[data-dm-target="filter-text"]', this.wrapper);
            this.textInput.on('change', function(event) {
                if (that.text != $(event.target).val()) {
                    that.text = $(event.target).val();
                    that.onChange();
                }
                return false;
            });
            this.textInput.on('keyup', function(event) {
               that.handleTextChange(event);
            });
            $('.dm-box .dm-text-search [type="submit"]').on('click', function(event) {
                        that.onChange();
            });
            $('[data-dm-target="reset-filter"]', this.wrapper).on('click', function(event) {
                that.resetState();
                that.onChange();
                return false;
            });
            // Adjust subcategories visibility
            $('.dm-filter-categories [data-dm-target="filter-toggler"]').on('click', function(event) {
                if (that.categoryFilter.is(':visible')) {
                    that.hideCategories();
                } else {
                    $('.dm-filter-categories [data-dm-target="filter-toggler-header"]').removeClass('dm-active').html($(event.target).siblings('h2').html());
                    $(event.target).siblings('ul.dm-filter').show(300);
                }
            });
            $('.dm-filter-card-types [data-dm-target="filter-toggler"]').on('click', function(event) {
                if (that.cardTypeFilter.is(':visible')) {
                    that.hideCardTypes();
                } else {
                    $('.dm-filter-card-types [data-dm-target="filter-toggler-header"]').removeClass('dm-active').html($(event.target).siblings('h2').html());
                    $(event.target).siblings('ul.dm-filter').show(300);
                }
            });
            // Wrap for smaller resolutions.
            $(window).resize(function() {
                if ($(window).width() < 650 ) { // 650 matches breakpoint in CSS
                    that.updateCategoriesTogglerHeader();
                    if (that.categoryFilter.is(':visible')) {
                        that.hideCategories();
                    }
                    that.updateCardTypesTogglerHeader();
                    if (that.cardTypeFilter.is(':visible')) {
                        that.hideCardTypes();
                    }
                }
                that.setPlaceholderSearchField(localizer);
            });
            //Initialize placeholder of search field
            this.setPlaceholderSearchField(localizer);
            // Initialize suggestions handler.
            this.initSuggestions();
            // Store available card types.
            $('[data-dm-target="filter-card-types-card-type"]').each(function (index, element) {
                that.cardTypes.push($(element).data('dm-card-type'));
            });
        },

        /**
         * Set placeholder value for search field
         **/
        setPlaceholderSearchField: function(localizer) {
            var message = localizer.message("searchPlaceholderLong");
            if($(window).width() < 650)
            message = localizer.message("searchPlaceholderShort");

            $(".dm-text-search .dm-filter-text").attr("placeholder", message);
        },

        /**
         * Handle change of the text search.
         */
        handleTextChange: function(event) {
            var that = this;
            var code = event.keyCode || event.which;
            var text = this.textInput.val().trim();
            // Stop prevous search timeout.
            clearTimeout(this.searchTimeout);
            // Do not perform search if the text didn't change.
            if (this.text == text) {
                return false;
            }
            // On enter search immediatelly
            if (code == 13) {
                this.text = text;
                this.onChange();
                return false;
            }
            // Do not perform search if the string is too short.
            if (text.length < 3) {
                return false;
            }
            // Postpone the live search.
            this.searchTimeout = setTimeout(function() {
                if (text == that.textInput.val().trim()) {
                    that.text = that.textInput.val().trim();
                    that.onChange();
                }
            }, 2000 /* 2s */);
        },

        /**
         * Set state based on query.
         * @param query Query object.
         */
        setState: function(query) {
            if (query && query.text) {
                this.setText(query.text);
            }
            if (query && query.categoryId) {
                this.setSelectedCategory(query.categoryId);
                this.updateCategoriesTogglerHeader();
            }
            if (query && query.cardType) {
                this.setSelectedCardType(query.cardType);
                this.updateCardTypesTogglerHeader();
            }
        },

        /**
         * Get all available card types for searching.
         */
        getAvailableCardTypes: function() {
            return this.cardTypes;
        },

        /**
         * Set searched text.
         */
        setText: function(text) {
            this.text = text;
            this.textInput.val(text);
        },

        /**
         * Reset state of the searching context to default values.
         */
        resetState: function() {
            this.setText(null);
            this.setSelectedCategory(null);
            this.setSelectedCardType(null);
        },

        /**
         * Set selected category and update state of the finder.
         * @param categoryId Identifier of the category.
         */
        setSelectedCategory: function(categoryId) {
            this.categoryId = categoryId;
            // If no category is selected, hide all and remove active class.
            if (this.categoryId == null) {
                $('.dm-filter-item-category').removeClass('dm-active');
                $('.dm-filter-item-category').children('a').data('dm-filter-open', false);
                $(".dm-filter-categories-child").hide(300);
                return;
            }
            // Find elemenets of category and parent category.
            var categoryElement = $('[data-dm-category-id="' + this.categoryId + '"]', this.categoryFilter);
            // Set category to toggler.
            var parentCategoryElement = null;
            if (categoryElement.data('dm-parent-category-id')) {
                parentCategoryElement = $('[data-dm-category-id="' + categoryElement.data('dm-parent-category-id')  + '"]', this.categoryFilter);
            }
            //Remove active class from all items.
            $('.dm-filter-item-category').removeClass('dm-active');
            // Colorize category.
            categoryElement.parent('li').addClass('dm-active');
            // Colorize parent category
            if (parentCategoryElement) {
                parentCategoryElement.parent('li').addClass('dm-active');
            }
            // Handle category expansion.
            // XXX: Tohle se mi vubec nelibi a chtělo zjednodušit.
            if (!parentCategoryElement && categoryElement.siblings('ul') && !categoryElement.data('dm-filter-open')) {
                categoryElement.parent('li').siblings('li').children('ul').hide(300);
                categoryElement.parent('li').siblings('li').children('a').data('dm-filter-open', false)
                categoryElement.data('dm-filter-open', true);
                categoryElement.siblings('ul').show(300);
            } else if (parentCategoryElement && parentCategoryElement.siblings('ul') && !parentCategoryElement.data('dm-filter-open')) {
                parentCategoryElement.parent('li').addClass('dm-active');
                parentCategoryElement.parent('li').siblings('li').children('ul').hide(300);
                parentCategoryElement.parent('li').siblings('li').children('a').data('dm-filter-open', false)
                parentCategoryElement.data('dm-filter-open', true);
                parentCategoryElement.siblings('ul').show(300);
            }
        },

        /**
         * Set selected card type and update displayed state of the finder.
         * @param cardType Card type to be set.
         */
        setSelectedCardType: function(cardType) {
            this.cardType = cardType;
            $('.dm-filter-item-card-type').removeClass('dm-active');
            if (this.cardType == null) {
                return;
            }
            $('[data-dm-card-type="' + this.cardType + '"]', this.cardTypeFilter).parent().addClass('dm-active');
        },

        /**
         * Update header for responsive toggler of card types.
         */
        updateCardTypesTogglerHeader: function() {
            if (this.cardType != null) {
                var cardTypeElement =  $('[data-dm-card-type="' + this.cardType + '"]', this.cardTypeFilter);
                $('.dm-filter-card-types [data-dm-target="filter-toggler-header"]').addClass('dm-active').html(cardTypeElement.html());
            } else {
                $('.dm-filter-card-types [data-dm-target="filter-toggler-header"]').removeClass('dm-active').html($('.dm-filter-card-types h2').html());
            }
        },

        /**
         * Update header for responsive toggler of categories.
         */
        updateCategoriesTogglerHeader: function() {
            if (this.categoryId != null) {
                var categoryElement =  $('[data-dm-category-id="' + this.categoryId + '"]', this.categoryFilter);
                $('.dm-filter-categories [data-dm-target="filter-toggler-header"]').addClass('dm-active').html(categoryElement.html());
            } else {
                $('.dm-filter-categories [data-dm-target="filter-toggler-header"]').removeClass('dm-active').html($('.dm-filter-categories h2').html());
            }
        },

        /**
         * Hide card types filter list.
         */
        hideCardTypes: function() {
             this.updateCardTypesTogglerHeader();
             this.cardTypeFilter.hide({duration: 300, done: function(event) {
                 $(this).css('display', ''); // Necessary to respect responsive behaviour of hiding.
             }});
        },

        /**
         * Hide categories filter list.
         */
        hideCategories: function() {
            this.updateCategoriesTogglerHeader();
            this.categoryFilter.hide({duration: 300, done: function(event) {
                $(this).css('display', ''); // Necessary to respect responsive behaviour of hiding.
            }});
        },

        /**
         * Initialize autocomplete functionality.
         */
        initSuggestions: function() {
            var that = this;
            $('[data-dm-target="filter-text"]').autocomplete({
                minLength: 1,
                delay: 200,
                source: this.ajaxUrl + "?action=suggestions",
                select: function (event, data) {
                    // Seach immediatelly when item is selected.
                    that.setText(data.item.text);
                    that.onChange();
                    return false;
                },
                focus: function(event, ui) {
                    // Do not perform searches when user is selecting a value.
                },
                open: function (event, ui) {
                    // Fix of width. Needed for good responsive behavior.
                    $(".dm-suggestions").css("width", $(event.target).innerWidth() +"px" );
                    return false;
                },
                messages: {
                    noResults: '',
                    results: function() {}
                },
                appendTo: "#dm-discount-finder .dm-text-search-options"
            }).data("ui-autocomplete")._renderItem = function (ul, item) {
                ul.addClass("dm-suggestions");
                return $("<li class=\"dm-suggestion\">")
                    .data("item.autocomplete", item)
                    .append($("<a></a>").text(item.text))
                    .appendTo(ul);
            };
            $('[data-dm-target="filter-text"]').keyup(function(event) {
                var code = event.keyCode || event.which;
                if (code == 13) {
                    // Close autocomplete on enter
                    $(this).autocomplete("close");
                }
            });
        },

        /**
         * Function executed with every change of the criteria.
         * XXX: Might be better to implement proper event handling.
         */
        onChange: function() {},

        /**
         * Get current search criteria as resolved by the API.
         */
        getSearchCriteria: function() {
             return {
                 'text': this.text,
                 'categoryIds': this.categoryId ? [this.categoryId] : null,
                 'cardTypes': this.cardType ? [this.cardType] : this.cardTypes
             }
        },

        /**
         * Get current search params as resolved by the application from search .
         */
        getSearchParams: function() {
             return {
                 'text': this.text,
                 'categoryId': this.categoryId,
                 'cardType': this.cardType
             }
        }
    });

    /**
     * Pagination handler. Is responsible for rendering of pagination
     * and storage of pagination status.
     */
    window.DmPagination = function(wrapper) {
        this.constructor.call(this, wrapper);
    }
    $.extend(DmPagination.prototype, {

        surroundings: 2,

        page: 1,
        pageSize: 10,
        resultsDisplayed: 0,
        count: 0,
        numOfPages: 1,

        pagesWrapper: null,
        moreResultsButton: null,

        /**
         * Construct the pagination object.
         * @pararm wrapper Wrapper of the finder plugin.
         */
        constructor: function(wrapper) {
            this.init(wrapper);
        },

        /**
         * Set state based on query.
         * @param query Query object.
         */
        setState: function(query) {
            if (query && query.page) {
                this.page = query.page;
                this.updatePage(this.page);
            }
        },

        /**
         * Initialize pagination.
         */
        init: function(wrapper) {
            var that = this;
            if (null == wrapper) {
                this.pagesWrapper = $('[data-dm-target="pagination-pages"]');
            } else {
                this.pagesWrapper = wrapper;
            }
            this.moreResultsButton = $('[data-dm-target="pagination-load-more"]');
            // Append listeners when page is changed.
            this.pagesWrapper.on('click', function(event) {
                if ($(event.target).data("dm-pagination-page") &&
                        that.page != $(event.target).data("dm-pagination-page")) {
                    that.updatePage($(event.target).data("dm-pagination-page"));
                }
                return false;
            });
            // Append listeners when more results are requested.
            this.moreResultsButton.on('click', function(event) {
                // TODO: rewrite the condition to use only page
                var resultsSkipped = (that.page - (that.resultsDisplayed / that.pageSize)) * that.pageSize;
                if ((that.resultsDisplayed + resultsSkipped) >= that.count) {
                    return;
                }
                that.onMoreResults((that.page - 1) * that.pageSize + that.resultsDisplayed, that.pageSize);
                that.resultsDisplayed += that.pageSize;
                that.page = that.page + 1;
                that.updateMoreResultsVisibility();
                // Re-render pages list with actual page.
                that.renderPages();
                return false;
            });
        },

        updateMoreResultsVisibility: function() {
            var resultsSkipped = (this.page - (this.resultsDisplayed / this.pageSize)) * this.pageSize;
            $('[data-dm-target="pagination-load-more-count"]', this.moreResultsButton).html(this.pageSize);
            if ((this.resultsDisplayed + resultsSkipped) >= this.count) {
                // Disable more button.
                this.moreResultsButton.addClass('dm-btn-disabled');
            } else {
                // Enable more button
                this.moreResultsButton.removeClass('dm-btn-disabled');
            }
        },

        /**
         * Set current page and update pagination.
         */
        updatePage: function(number) {
            this.page = number;
            this.resultsDisplayed = this.pageSize;
            this.onPageChange();
            this.renderPages();
        },

        /**
         * Render list of pages.
         */
        renderPages: function() {
            var maxPagesBefore = this.surroundings;
            var maxPagesAfter = this.surroundings

            if ((this.numOfPages - this.page) < this.surroundings) {
                maxPagesBefore += this.surroundings - (this.numOfPages - this.page);
            }
            if ((this.page < this.surroundings)) {
                maxPagesAfter += this.surroundings - this.page;
            }
            var html = '';
            html = this.renderPage(1);
            if (2 < (this.page - maxPagesBefore)) {
                html += '<span class="dm-pagination-separator">...</span>';
            }
            for (var i = (this.page - maxPagesBefore); i < (this.page + maxPagesAfter +1); i++) {
                if (i > 1 && i < this.numOfPages) {
                    html += this.renderPage(i);
                }
            }
            if ((this.page + maxPagesAfter) < this.numOfPages - 1) {
                html += '<span class="dm-pagination-separator">...</span>';
            }
            if (this.numOfPages > 1) {
                html += this.renderPage(this.numOfPages);
            }
            this.pagesWrapper.html(html);
        },

        /**
         * Render single page item.
         */
        renderPage: function(number) {
            return '<a class="dm-pagination-page ' + (this.page == number ? 'dm-active' : '') + '" data-dm-pagination-page="' + number + '">' + number + '</a>';
        },

        /**
         * Handle page change event.
         */
        onPageChange: function() {},

        /**
         * Handle more results button.
         */
        onMoreresults: function() {},

        /**
         * Get currently rendered page options.
         */
        getPagingParams: function() {
            return {
                'offset' : (this.page - 1) * this.pageSize ,
                'limit' : this.pageSize
            };
        },

        /**
         * Reset pagination to the first page.
         */
        reset: function() {
            this.page = 1;
            this.resultsDisplayed = this.pageSize;
            this.updateMoreResultsVisibility();
        },

        /**
         * Change page size that should be used for paging.
         * @param Set page size that should be used for pagination.
         */
        setPageSize: function(pageSize) {
            this.pageSize = pageSize;
        },

        /**
         * Set number of results.
         */
        setCount: function(count) {
            this.count = count;
            this.numOfPages = Math.ceil(count / this.pageSize);
            this.updateMoreResultsVisibility();
        }

    });

    /**
     * Sorting handler. Responsible for handling results sorting
     * and storage of search parameters.
     */
    window.DmSorting = function(wrapper, localizer) {
        this.constructor.call(this, wrapper, localizer);
    }
    $.extend(DmSorting.prototype, {

        localizer: null,

        location: null,
        seed: null,

        sortKey: null,
        sortOrder: null,

        sortKeySelect: null,
        sortOrderCheckbox: null,

        /**
         * Construct the sorting object.
         * @param wrapper Wrapper of the finder plugin.
         * @param query Array representaion of the query string.
         */
        constructor: function(wrapper, localizer) {
            this.init(wrapper);
            this.localizer = localizer;
        },

        /**
         * Request user location.
         */
        requestLocation: function(callback, errorCallback) {
             if (navigator.geolocation) {
                 navigator.geolocation.getCurrentPosition(callback, errorCallback);
             } else {
                 errorCallback();
             }
         },

        /**
         * Set state based on query.
         * @param query Query object.
         */
        setState: function(query) {
            if (query && query.sortKey) {
                this.sortKey = query.sortKey;
                this.sortKeySelect.val(this.sortKey);
            }
            if (query && query.seed) {
                this.seed = query.seed;
            }
            if (query && query.sortOrder) {
                this.sortOrder = query.sortOrder;
                if (this.sortOrder == 'desc') {
                    this.sortOrderCheckbox.prop('checked', true);
                }
            }
        },

        /**
         * initialize the sorting.
         * @param Wrapper of the finder plugin.
         */
        init: function(wrapper) {
            var that = this;
            this.sortKeySelect = $('[data-dm-target="sort-key-select"]', wrapper);
            this.seed = Math.floor(Math.random() * 10000);  // Initial seed for searching.
            this.sortKeySelect.on('change', function(event) {
                if (that.sortKey != $(event.target).val()) {
                    that.sortKey = $(event.target).val();
                     if (that.sortKey == "distance") {
                         that.requestLocation(function(position) {
                             that.location = position.coords.latitude + "," + position.coords.longitude;
                             that.onChange();
                         },
                         function() {
                            that.sortKey = "";
                            alert(that.localizer.message("sort_location_missing"));
                            that.onChange();
                         });
                     } else if (that.sortKey == "createdOn" || that.sortKey == "views") {
                         // Sort desc by default for createdOn and views
                         that.sortOrder = "desc";
                         that.sortOrderCheckbox.prop('checked', true);
                         that.onChange();
                     } else {
                         that.onChange();
                     }
                    return false;
                }
            });
            this.sortOrderCheckbox = $('[data-dm-target="sort-order-switch-checkbox"]', this.wrapper);
            this.sortOrderCheckbox.on('change', function(event) {
                if ($(event.target).prop('checked')) {
                    that.sortOrder = 'desc';
                } else {
                    that.sortOrder = 'asc';
                }
                that.onChange();
            });
        },

        /**
         * Handle the on change event. Can be replaced externally to perform event handling.
         * XXX: Might be better to implement proper event handling.
         */
        onChange: function() {},

        /**
         * Get current sort params.
         */
        getSortParams: function() {
             return {
                 'sortKey' : this.sortKey,
                 'sortOrder' : this.sortOrder,
                 'location' : this.location,
                 'seed' : this.seed
             }
        }

    });


    /**
     * Discount manager client. Is responsible for contacting
     * DM and fetching the data. This class should be always used
     * when calling the discount manager.
     */
    window.DmClient = function(ajaxUrl) {
        this.constructor.call(this, ajaxUrl);
    }
    $.extend(DmClient.prototype, {

        ajaxUrl: null,

        /**
         * Construct the sorting object.
         * @param ajaxUrl Url of the php ajax handler.
         * @param query Array representaion of the query string.
         */
        constructor: function(ajaxUrl) {
            if (!ajaxUrl) {
                throw new Error("ajaxUrl has to be specified.");
            }
            this.ajaxUrl = ajaxUrl;
        },

        /**
         * Get providers from DM.
         * @param requestParams Request parameters.
         * @param callback Callback function instance.
         */
        getProviders: function(requestParams, callback) {
            if (this.ajaxUrl !== 'not initialized') {
                $.ajax(this.ajaxUrl, {
                    // Make request to the test_providers action.
                    data: $.extend(requestParams, {
                        action: "providers"
                    }),
                    dataType: "json",
                    success: callback,
                    error: function(err) {
                        throw new Error('Failed to get providers from DM.' + err);
                    }
                });
            }
        },

        getProvider: function(requestParams, callback) {
            if (this.ajaxUrl !== 'not initialized') {
                $.ajax(this.ajaxUrl, {
                    data: $.extend(requestParams, {
                        action: "provider"
                    }),
                    dataType: "json",
                    success: callback,
                    error: function (err) {
                        throw new Error("Failed to get provider with id '" + requestParams.providerId + "'." + err);
                    }
                });
            }
        },

        viewProvider: function(requestParams) {
            if (this.ajaxUrl !== 'not initialized') {
                $.ajax(this.ajaxUrl, {
                    data: $.extend(requestParams, {
                        action: "add_provider_view"
                    }),
                    dataType: "json",
                    type: "POST",
                    error: function (err) {
                        throw new Error("Failed to view provider with id '" + requestParams.providerId + "'." + err);
                    }
                });
            }
        },

        /**
         * Get branches from DM.
         * @param requestParams Request parameters.
         * @param callback Callback function instance.
         */
        getBranches: function(requestParams, callback) {
            $.ajax(this.ajaxUrl, {
                data: $.extend(requestParams, {
                    action: "branches"
                }),
                dataType: "json",
                success: callback,
                error: function(err) {
                    throw new Error('Failed to get branches from DM.' + err);
                }
            });
        },

        getComments: function(providerId, offset, limit, callback) {
            $.ajax(
                    this.ajaxUrl,
                    {
                        data: {
                            action: 'comments',
                            providerId: providerId,
                            offset: offset,
                            limit: limit,
                        },
                        dataType: 'json',
                        success: callback,
                        error: function(err) {
                            throw new Error('Failed to get comment: ' + err);
                        }
                    }
            );
        },

        rateProvider: function(id, value, callback) {
            $.ajax(
                this.ajaxUrl,
                {
                    data: {
                        action: 'rate_provider',
                            providerId: id,
                        value: value,
                    },
                    dataType: 'json',
                    type: 'post',
                    success: callback,
                    error: function(err) {
                        throw new Error('Failed to rate provider: ' + err);
                    }
                }
            );
        },

        comment: function(id, username, cardnumber, text, ratingId, callback, forbiddenCallback) {
            $.ajax(
                this.ajaxUrl,
                {
                    data: {
                        action: 'comment',
                            providerId: id,
                        username: username,
                        cardnumber: cardnumber,
                        text: text,
                        rating: ratingId,
                    },
                    dataType: 'json',
                    type: 'post',
                    success: callback,
                    error: function(err) {
                        if (err.status == 403) {
                            forbiddenCallback();
                        }
                        throw new Error('Failed to comment on provider: ' + err);
                    }
                }
            );
        },
    });

  }) ( jQuery );
